@@ -19,6 +19,7 @@
 
 #include <stdio.h>
 #include <sys/types.h>
+#include <ctype.h>
 
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -26,11 +27,20 @@
 #include <signal.h>
 #include <netdb.h>
 
+#include <string.h>
+#include <stdlib.h>
+#include <unistd.h>
+
+#ifdef WIN32
+#  include <winsock.h>
+#endif
+
 
 #include <common/portability.h>
 #include "msql_priv.h"
 
-char *ident_client ();
+
+
 
 #define	ALLOW	1
 #define REJECT	2
@@ -109,18 +119,15 @@ int msqlLoadAcl(verbose)
 	/*
 	** Open the acl file
 	*/
-#ifndef OLIS
-	(void)sprintf(path,"/etc/msql.acl");
-#else
-	(void)sprintf(path,"%s/msql.acl", msqlHomeDir);
-#endif
+	(void)sprintf(path,"%s/msql.acl", 
+		(char *)msqlGetCharConf("general", "inst_dir"));
 	fp = fopen(path,"r");
 	if (!fp)
 	{
 		if (verbose)
 		{
-			perror("Warning : Couldn't open ACL file");
-			fprintf(stderr,"Without an ACL file global access is Read/Write\n\n");
+			fprintf(stderr,
+			   "\nWarning : No ACL file.  Using global read/write access.\n");
 		}
 		sprintf(packet,"-1:Warning - Couldn't open ACL file\n");
 		return(-1);
@@ -448,41 +455,6 @@ int msqlLoadAcl(verbose)
 }
 
 
-dumpAcc(acc)
-	acc_t	*acc;
-{
-	while(acc)
-	{
-		printf("\t\t%s %s\n",(acc->access==ALLOW)?"Yes":"No ",
-			acc->name);
-		acc = acc->next;
-	}
-}
-
-
-static dumpAcl()
-{
-	acl_t	*curAcl;
-	tlist_t	*curT;
-	acc_t	*curAcc;
-
-	curAcl = aclHead;
-	while(curAcl)
-	{
-		printf("\nACL's for Database %s\n",curAcl->db);
-		printf("\tRead access :-\n");
-		dumpAcc(curAcl->read);
-		printf("\tWrite access :-\n");
-		dumpAcc(curAcl->write);
-		printf("\tHost access :-\n");
-		dumpAcc(curAcl->host);
-
-		curAcl = curAcl->next;
-	}
-}
-
-
-
 static int matchToken(pattern,tok)
 	char	*pattern,
 		*tok;
@@ -624,9 +596,7 @@ int msqlCheckAccess(db,info)
 	struct  sockaddr *local, 
 		*remote;
 	acl_t	*curAcl;
-	acc_t	*curAcc;
-	int	res,
-		perms;
+	int	perms;
 
 
 	host = info->host;
@@ -697,14 +667,6 @@ int msqlCheckAccess(db,info)
 	/*
 	** Now perform any options for this connection
 	*/
-	if (host && matchTextList(curAcl->option,"ident"))
-	{
-	    extern char *ident_client();
-	    char *remoteUser;
-	    remoteUser = ident_client(remote, local);
-	    if (!remoteUser || strcmp(remoteUser, user) != 0)
-		return(NO_ACCESS);
-	}
 
 	
 	return(perms);
@@ -712,7 +674,7 @@ int msqlCheckAccess(db,info)
 
 
 
-static freeAcc(head)
+static void freeAcc(head)
 	acc_t	*head;
 {
 	register acc_t	*curAcc,
@@ -727,7 +689,7 @@ static freeAcc(head)
 	}
 }
 
-static freeTlist(head)
+static void freeTlist(head)
 	tlist_t	*head;
 {
 	register tlist_t	*curText,
@@ -743,7 +705,7 @@ static freeTlist(head)
 }
 
 
-static freeAcls()
+static void freeAcls()
 {
 	register acl_t	*curAcl,
 			*prevAcl;
@@ -763,7 +725,7 @@ static freeAcls()
 	aclHead = NULL;
 }
 
-reloadAcls(sock)
+void msqlServerReloadAcls(sock)
 	int	sock;
 {
 	freeAcls();
@@ -771,7 +733,7 @@ reloadAcls(sock)
 }
 
 
-msqlSetPerms(perms)
+void msqlSetPerms(perms)
 	int	perms;
 {
 	accessPerms = perms;
@@ -792,23 +754,35 @@ int msqlCheckLocal(info)
 		*user;
 	struct  sockaddr *local, 
 		*remote;
-	acl_t	*curAcl;
-	acc_t	*curAcc;
-	int	res,
-		perms;
-
 
 	host = info->host;
 	user = info->user;
 	local = (struct sockaddr *)&(info->local);
 	remote = (struct sockaddr *)&(info->remote);
-	if ( (!info->host || strcmp(host,"localhost") == 0) &&
-	     strcmp(info->user,ROOT) == 0)
+
+#ifdef WIN32
+	if (local->sa_data[2] == 127 &&
+	    local->sa_data[3] == 0 &&
+	    local->sa_data[4] == 0 &&
+	    local->sa_data[5] == 1)
 	{
+		/* Localhost connection */
 		return(1);
 	}
+	return(0);
+#else
+	if ( (!info->host || strcmp(host,"localhost") == 0))
+	{
+		if (strcmp(info->user,
+			(char *)msqlGetCharConf("general","admin_user"))!=0)
+		{
+			return(0);
+		}
+	}
 	else
 	{
 		return(0);
 	}
+	return(1);
+#endif
 }
