@@ -32,7 +32,7 @@
 
    /* CLIENT LIBRARY MESSAGES */
 
-#  define SOCKET_ERROR        	"Can't create UNIX socket"
+#  define SOCK_ERROR        	"Can't create UNIX socket"
 #  define CONNECTION_ERROR    	"Can't connect to local MSQL server"
 #  define IPSOCK_ERROR        	"Can't create IP socket"
 #  define UNKNOWN_HOST        	"Unknown MSQL Server Host (%s)"
@@ -53,15 +53,15 @@
 #  define NO_DB_ERROR		"No Database Selected"
 #  define PERM_DENIED_ERROR	"Permission denied"
 #  define UNKNOWN_COM_ERROR	"Unknown command"
-#  define BAD_DIR_ERROR		"Can't open directory \"%s\""
+#  define BAD_DIR_ERROR		"Can't open directory \"%s\" (%s)"
 #  define BAD_TABLE_ERROR	"Unknown table \"%s\""
-#  define TABLE_READ_ERROR	"Error reading table \"%s\" definition"
+#  define TABLE_READ_ERROR	"Error reading table \"%s\" definition (%s)"
 #  define TMP_MEM_ERROR		"Out of memory for temporary table"
-#  define TMP_CREATE_ERROR	"Couldn't create temporary table"
-#  define DATA_OPEN_ERROR	"Couldn't open data file for %s"
-#  define KEY_OPEN_ERROR	"Couldn't open key file for %s"
-#  define STACK_OPEN_ERROR	"Couldn't open stack file for %s"
-#  define WRITE_ERROR		"Data write failed"
+#  define TMP_CREATE_ERROR	"Couldn't create temporary table (%s)"
+#  define DATA_OPEN_ERROR	"Couldn't open data file for %s (%s)"
+#  define KEY_OPEN_ERROR	"Couldn't open key file for %s (%s)"
+#  define STACK_OPEN_ERROR	"Couldn't open stack file for %s (%s)"
+#  define WRITE_ERROR		"Data write failed (%s)"
 #  define KEY_WRITE_ERROR	"Write of key failed"
 #  define SEEK_ERROR		"Seek into data table failed!"
 #  define KEY_SEEK_ERROR	"Seek into key table failed!"
@@ -69,7 +69,9 @@
 #  define FIELD_COUNT_ERROR	"Too many fileds in query"
 #  define TYPE_ERROR		"Literal value for \'%s\' is wrong type"
 #  define BAD_FIELD_ERROR	"Unknown field \"%s.%s\""
+#  define BAD_ORD_FIELD_ERROR	"Bad order field. Field \"%s.%s\" was not selected"
 #  define BAD_FIELD_2_ERROR	"Unknown field \"%s\""
+#  define BAD_ORD_FIELD_2_ERROR	"Bad order field. Field \"%s\" was not selected"
 #  define COND_COUNT_ERROR	"Too many fields in condition"
 #  define ORDER_COUNT_ERROR	"Too many fields in order specification"
 #  define BAD_LIKE_ERROR	"Evaluation of LIKE clause failed"
@@ -83,14 +85,24 @@
 #  define TABLE_WIDTH_ERROR	"Too many fields in table (%d Max)"
 #  define CATALOG_WRITE_ERROR	"Error writing catalog"
 #  define KEY_CREATE_ERROR	"Creation of key table failed"
-#  define DATA_FILE_ERROR	"Error creating table file for \"%s\""
+#  define DATA_FILE_ERROR	"Error creating table file for \"%s\" (%s)"
 #  define BAD_TABLE_ERROR	"Unknown table \"%s\""
 #  define NO_VALUE_ERROR	"No value specified for field '%s'"
 #  define NON_UNIQ_ERROR	"Field '%s' not unique"
-#  define KEY_UNIQ_ERROR	"Non unique key value in field '%s'"
+#  define KEY_UNIQ_ERROR	"Non unique value for unique index"
 #  define UNSELECT_ERROR	"Reference to un-selected table \"%s\""
 #  define UNQUAL_JOIN_ERROR	"Unqualified field \"%s\" in join"
-#  define SIZE_ERROR		"Value for field \'%s\' is too large"
+#  define TEXT_COND_ERROR	"Can't use TEXT fields in comparison\n"
+#  define TEXT_REGEX_ERROR	"Can't use TEXT fields in LIKE comparison\n"
+#  define SYSVAR_ERROR		"Unknown system variable \"%s\""
+
+#  define VALUE_SIZE_ERROR	"Value for \"%s\" is too large"
+#  define NULL_INDEX_ERROR	"Index field \"%s\" cannot be NULL"
+#  define NULL_COND_ERR		"Index condition for \"%s\" cannot be NULL"
+#  define NULL_JOIN_COND_ERR	"Index condition for \"%s\" in join cannot be NULL"
+
+#  define DATE_ERROR		"Invalid date format \"%s\""
+#  define TIME_ERROR		"Invalid time format \"%s\""
 
 #endif
 
@@ -106,7 +118,7 @@
 
    /* CLIENT LIBRARY MESSAGES */
 
-#  define SOCKET_ERROR 		"Kann UNIX-Socket nicht anlegen"
+#  define SOCK_ERROR 		"Kann UNIX-Socket nicht anlegen"
 #  define CONNECTION_ERROR	"Keine Verbindung zu lokalem MSQL Server"
 #  define IPSOCK_ERROR   	"Kann IP-Socket nicht anlegen"
 #  define UNKNOWN_HOST   	"Unbekannter MSQL Server Host (%s)"
@@ -158,14 +170,17 @@
 #  define TABLE_WIDTH_ERROR	"Too many fields in table (%d Max)"
 #  define CATALOG_WRITE_ERROR	"Error writing catalog"
 #  define KEY_CREATE_ERROR	"Creation of key table failed"
-#  define DATA_FILE_ERROR	"Error creating table file for \"%s\""
+#  define DATA_FILE_ERROR	"Error creating table file for \"%s\" (%s)"
 #  define BAD_TABLE_ERROR	"Unknown table \"%s\""
 #  define NO_VALUE_ERROR	"No value specified for field '%s'"
 #  define NON_UNIQ_ERROR	"Field '%s' not unique"
 #  define KEY_UNIQ_ERROR	"Non unique key value in field '%s'"
 #  define UNSELECT_ERROR	"Reference to un-selected table \"%s\""
 #  define UNQUAL_JOIN_ERROR	"Unqualified field \"%s\" in join"
-#  define SIZE_ERROR		"Value for field \'%s\' is too large"
+
+#  define VALUE_SIZE_ERROR	"Value for \"%s\" is too large"
+#  define NULL_INDEX_ERROR	"Index field \"%s\" cannot be NULL"
+#  define NULL_COND_ERR		"Index condition for \"%s\" cannot be NULL"
 
 #endif
 
