@@ -22,14 +22,27 @@
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-#include <arpa/inet.h>
-
 #include <signal.h>
 #include <netdb.h>
-#include <pwd.h>
 #include <stdlib.h>
-#include <varargs.h>
-#include <string.h>
+
+#ifdef OS2
+#  include <arpa/nameser.h>
+#  define ANSI_ARGS
+#else
+#  include <arpa/inet.h>
+#  include <unistd.h>
+#  include <stdlib.h>
+#  include <string.h>
+#endif
+
+#ifdef WIN32
+#  include <winsock.h>
+#endif
+
+#if !defined(WIN32) && !defined(OS2)
+#  include <pwd.h>
+#endif
 
 #include "common/portability.h"
 
@@ -37,13 +50,19 @@
 #  include <sys/un.h>
 #endif
 
+#ifdef ANSI_ARGS
+#  include <stdarg.h>
+#else
+#  include <varargs.h>
+#endif
+
 
 #include "common/site.h"
 
 #include "msql_priv.h"
 #include "errmsg.h"
 
-#define _LIB_SOURCE
+#define	_MSQL_SERVER_SOURCE
 #include "msql.h"
 
 #ifndef	INADDR_NONE
@@ -63,10 +82,20 @@ static	m_data *tmpDataStore = NULL,
 		*fieldData = NULL;
 
 
-char	msqlErrMsg[160];
+#define	ERR_BUF_LEN	200
+char	msqlErrMsg[ERR_BUF_LEN];
+extern	int	msqlConfigLoaded;
 
 RETSIGTYPE	(*oldHandler)();
+#ifdef WIN32
+RETSIGTYPE pipeHandler(int);
+#endif
+
+#ifdef OS2
+static void msqlDebug( int module, ... );
+#else
 static void msqlDebug();
+#endif
 
 #define	resetError()	(void)bzero(msqlErrMsg,sizeof(msqlErrMsg))
 #define chopError()	{ char *cp; cp = msqlErrMsg+strlen(msqlErrMsg) -1; \
@@ -79,6 +108,27 @@ extern	char	*packet;
 
 
 
+#ifdef OS2
+/**************************************************************************
+**	_msqlGetErrMsg
+**
+**	Purpose	: 
+**	Args	: 
+**	Returns	: 
+**	Notes	: 
+*/
+char *APIENTRY msqlGetErrMsg(msg)
+	char	*msg;
+{
+	if (!msg)
+		return((char *)&msqlErrMsg);
+	strncpy(msg, msqlErrMsg, strlen(msqlErrMsg));
+	return(msg);
+}
+#endif
+
+
+
 /**************************************************************************
 **	_ msqlInitDebug
 **
@@ -103,7 +153,7 @@ static void msqlInitDebug()
 		*tmp,
 		*tok;
 
-	env = getenv("MINERVA_DEBUG");
+	env = getenv("MSQL_DEBUG");
 	if(env)
 	{
 		tmp = (char *)strdup(env);
@@ -111,7 +161,7 @@ static void msqlInitDebug()
 	else
 		return;
 	printf("\n-------------------------------------------------------\n");
-	printf("MINERVA_DEBUG found. libmsql started with the following:-\n\n");
+	printf("MSQL_DEBUG found. libmsql started with the following:-\n\n");
 	tok = (char *)strtok(tmp,":");
 	while(tok)
 	{
@@ -150,17 +200,26 @@ static void msqlInitDebug()
 **	Notes	: 
 */
 
+#ifdef ANSI_ARGS
+static void msqlDebug(int module, ...)
+#else
 static void msqlDebug(va_alist)
 	va_dcl
+#endif
 {
 		va_list args;
 	char	msg[1024],
 		*fmt;
-	int	module,
-		out = 0;
+
+#ifdef ANSI_ARGS
+	va_start(args, module);
+#else
+	int	module;
 
 	va_start(args);
-	module = (int) va_arg(args, int *);
+	module = (int) va_arg(args, int );
+#endif
+
 	if (! (module & debugLevel))
 	{
 		va_end(args);
@@ -172,8 +231,8 @@ static void msqlDebug(va_alist)
         	return;
 	(void)vsprintf(msg,fmt,args);
 	va_end(args);
-	printf("[libmsql] %s",msg);
-	fflush(stdout);
+	fprintf(stderr,"[libmsql] %s",msg);
+	fflush(stderr);
 }
 
 
@@ -210,8 +269,11 @@ static void closeServer(sock)
 {
 	msqlDebug(MOD_API,"Server socket (%d) closed\n", sock);
 	shutdown(sock,2);
-	signal(SIGPIPE,oldHandler);
+#ifdef OS2
+	soclose(sock);
+#else
 	close(sock);
+#endif
 }
 
 
@@ -230,8 +292,6 @@ static void closeServer(sock)
 void msqlClose(sock)
 	int	sock;
 {
-	char	buf[6];
-
 	setServerSock(sock);
 	sprintf(packet,"%d:\n",QUIT);
 	writePkt(sock);
@@ -256,13 +316,18 @@ RETSIGTYPE pipeHandler(sig)
 {
 	msqlDebug(MOD_API,"Hit by pipe signal\n");
 	closeServer(curServerSock);
+#ifdef OS2
+	signal(SIGABRT,pipeHandler);
+#else
+	signal(SIGPIPE,pipeHandler);
+#endif
 	return;
 }
 
 
 
 
-static freeQueryData(cur)
+static void freeQueryData(cur)
 	m_data	*cur;
 {
 	m_data	*prev;
@@ -279,17 +344,16 @@ static freeQueryData(cur)
 		safeFree(cur->data);
 		prev = cur;
 		cur = cur->next;
-		safeFree(prev);
 		msqlDebug(MOD_MALLOC, "Query data row - free @ %X\n", prev);
+		safeFree(prev);
 	}
-	safeFree(cur);
 }
 
 
 
 
 
-static freeFieldList(fieldData)
+static void freeFieldList(fieldData)
 	m_fdata	*fieldData;
 {
 	m_fdata	*cur,
@@ -302,8 +366,8 @@ static freeFieldList(fieldData)
 		cur = cur->next;
 		safeFree(prev->field.table);
 		safeFree(prev->field.name);
-		safeFree(prev);
 		msqlDebug(MOD_MALLOC, "Field List Entry- free @ %X\n", prev);
+		safeFree(prev);
 	}
 }
 
@@ -323,8 +387,8 @@ void msqlFreeResult(result)
 {
 	freeQueryData(result->queryData);
 	freeFieldList(result->fieldData);
-	safeFree(result);
 	msqlDebug(MOD_MALLOC,"Result Handle - Free @ %X\n",result);
+	safeFree(result);
 }
 
 		
@@ -362,7 +426,7 @@ static m_fdata *tableToFieldList(data)
 		if (*curRow->data[4] == 'Y')
 			curField->field.flags |= NOT_NULL_FLAG;
 		if (*curRow->data[5] == 'Y')
-			curField->field.flags |= PRI_KEY_FLAG;
+			curField->field.flags |= UNIQUE_FLAG;
 		curRow = curRow->next;
 	}
 	return(head);
@@ -378,27 +442,39 @@ static m_fdata *tableToFieldList(data)
 **	Notes	: If host == NULL, localhost is used via UNIX domain socket
 */
 
-int msqlConnect(myhost)
-	char	*myhost;
+#if defined(OS2) || defined(WIN32)
+int msqlUserConnect();
+
+int msqlConnect(host)
+        char    *host;
+{
+        return msqlUserConnect( host, NULL );
+}
+
+int msqlUserConnect(host, user)
+        char    *host;
+        char    *user;
+#else
+int msqlConnect(host)
+        char    *host;
+#endif
 {
 	char	*cp,
-		*envVar,
 		*unixPort;
 	struct	sockaddr_in IPaddr;
-	char	*host = NULL;
 
 #ifdef HAVE_SYS_UN_H
 	struct	sockaddr_un UNIXaddr;
 #endif
-        struct  servent         *serv_ptr;
 	struct	hostent *hp;
 	u_long	IPAddr;
 	int	opt,
-		version,
 		sock,
+		version,
 		tcpPort;
 	struct	passwd *pw;
 
+
 	resetError();
 	initNet();
 	if (!debugInit)
@@ -406,13 +482,11 @@ int msqlConnect(myhost)
 		debugInit++;
 		msqlInitDebug();
 	}
-
-	if (myhost) {
-		host = (char *)malloc(strlen(myhost)+1);
-		if (!host)
-			return -1;
-		strcpy(host,myhost);
+	if (!msqlConfigLoaded)
+	{
+		msqlLoadConfigFile(NULL);
 	}
+
 	/*
 	** Grab a socket and connect it to the server
 	*/
@@ -420,7 +494,7 @@ int msqlConnect(myhost)
 #ifndef HAVE_SYS_UN_H
 	if (!host)
 	{
-		host = "localhost";
+		host = "127.0.0.1";
 	}
 #endif
 
@@ -428,18 +502,14 @@ int msqlConnect(myhost)
 	{
 #ifdef HAVE_SYS_UN_H
 		/* Shouldn't get in here with UNIX socks */
-	        unixPort = MSQL_UNIX_ADDR;
-        	if ((envVar = getenv("MSQL_UNIX_PORT")))
-        	{
-                	unixPort = envVar;
-        	}
+	        unixPort = msqlGetCharConf("general","unix_port");
 		strcpy(hostInfo,"Localhost via UNIX socket");
 		msqlDebug(MOD_API,"Server name = NULL.  Using UNIX sock(%s)\n",
 			unixPort);
 		sock = socket(AF_UNIX,SOCK_STREAM,0);
 		if (sock < 0)
 		{
-			sprintf(msqlErrMsg,SOCKET_ERROR);
+			sprintf(msqlErrMsg,SOCK_ERROR);
 			return(-1);
 		}
 		setServerSock(sock);
@@ -454,29 +524,18 @@ int msqlConnect(myhost)
 			sizeof(UNIXaddr))<0)
 		{
 			sprintf(msqlErrMsg,CONNECTION_ERROR);
+#ifdef OS2
+			soclose(sock);
+#else
 			close(sock);
+#endif
 			return(-1);
 		}
 #endif
 	}
 	else
 	{
-	        tcpPort = MSQL_PORT;
-        	if ((serv_ptr = getservbyname("msql", "tcp")))
-        	{
-                	tcpPort = ntohs(serv_ptr->s_port);
-        	}
-        	if ((envVar = getenv("MSQL_TCP_PORT")))
-        	{
-                	tcpPort = atoi(envVar);
-        	}
-		if ((cp=index (host, ':')))
-		{
-			*cp='\0';
-			cp++;
-			tcpPort = atoi(cp);
-		}
-
+	        tcpPort = msqlGetIntConf("general","tcp_port");
 		sprintf(hostInfo,"%s via TCP/IP",host);
 		msqlDebug(MOD_API,"Server name = %s.  Using TCP sock (%d)\n",
 			host, tcpPort);
@@ -510,7 +569,11 @@ int msqlConnect(myhost)
 				sprintf(msqlErrMsg,
 					UNKNOWN_HOST,
 					host);
+#ifdef OS2
+				soclose(sock);
+#else
 				close(sock);
+#endif
 				return(-1);
 			}
 			bcopy(hp->h_addr,&IPaddr.sin_addr, hp->h_length);
@@ -520,13 +583,20 @@ int msqlConnect(myhost)
 			sizeof(IPaddr))<0)
 		{
 			sprintf(msqlErrMsg,CONN_HOST_ERROR, host);
-			perror("Connect");
+#ifdef OS2
+			soclose(sock);
+#else
 			close(sock);
+#endif
 			return(-1);
 		}
 	}
 
+#ifdef OS2
+	oldHandler = signal(SIGABRT,pipeHandler);
+#else
 	oldHandler = signal(SIGPIPE,pipeHandler);
+#endif
 	msqlDebug(MOD_API,"Connection socket = %d\n",sock);
 
 	/*
@@ -549,7 +619,7 @@ int msqlConnect(myhost)
 
 	if (atoi(packet) == -1)  
 	{
-		if (cp = (char *)index(packet,':'))
+		if ((cp = (char *)index(packet,':')))
 		{
 			strcpy(msqlErrMsg,cp+1);
 			chopError();
@@ -573,7 +643,8 @@ int msqlConnect(myhost)
 	version = atoi(cp + 1);
 	if (version != PROTOCOL_VERSION) 
 	{
-		sprintf(msqlErrMsg, VERSION_ERROR, version, PROTOCOL_VERSION);
+		sprintf(msqlErrMsg, VERSION_ERROR, version, 
+			PROTOCOL_VERSION);
 		closeServer(sock);
 		return(-1);
 	}
@@ -583,32 +654,55 @@ int msqlConnect(myhost)
 	cp = (char *)index(cp+1,':');
 	if (cp)
 	{
-		msqlDebug(MOD_API,"Server greeting = '%s'\n",cp+1);
 		strcpy(serverInfo,cp+1);
+		if (*(serverInfo+strlen(cp+1)-1) == '\n')
+		{
+			*(serverInfo+strlen(cp+1)-1) = 0;
+		}
+		msqlDebug(MOD_API,"Server greeting = '%s'\n",cp+1);
 	}
 	else
 	{
 		strcpy(serverInfo,"Error in server handshake!");
 	}
-	if (*(serverInfo+strlen(cp+1)-1) == '\n')
-	{
-		*(serverInfo+strlen(cp+1)-1) = 0;
-	}
-
 
 	/*
 	** Send the username for this process for ACL checks
 	*/
+#if defined(OS2) || defined(WIN32)
+	if (!user || *user == '\0')
+	{
+		char	*psz;
+
+		psz = getenv("USER");
+		if (!psz)
+			psz = "unknown";
+		(void)sprintf( packet, "%s\n", psz );
+	}
+	else
+	{
+		(void)sprintf( packet,"x_%s\n", user );
+	}
+#else
+		
 	pw = getpwuid(geteuid());
 	if (!pw)
 	{
-		strcpy(msqlErrMsg,USERNAME_ERROR);
-		closeServer(sock);
-		return(-1);
+		char	*psz;
+
+		psz = getenv("USER");
+		if (!psz)
+		{
+			strcpy(msqlErrMsg,USERNAME_ERROR);
+			closeServer(sock);
+			return(-1);
+		}
+		(void)sprintf(packet,"%s\n",psz);
 	}
-	(void)sprintf(packet,"%s\n",pw->pw_name);
+	else
+		(void)sprintf(packet,"%s\n",pw->pw_name);
+#endif
 	writePkt(sock);
-	(void)bzero(packet,PKT_LEN);
 	if(readPkt(sock) <= 0)
 	{
 		closeServer(sock);
@@ -659,9 +753,6 @@ int msqlSelectDB(sock,db)
 	int	sock;
 	char	*db;
 {
-	int	res;
-
-
 
         msqlDebug(MOD_API,"Select Database = \"%s\"\n",db);
 
@@ -674,7 +765,6 @@ int msqlSelectDB(sock,db)
 
 	(void)sprintf(packet,"%d:%s\n",INIT_DB,db);
 	writePkt(sock);
-	(void)bzero(packet,PKT_LEN);
 	if(readPkt(sock) <= 0)
 	{
 		closeServer(sock);
@@ -735,7 +825,10 @@ m_result *msqlStoreResult()
 	{
 		return(NULL);
 	}
-	(void)bzero(tmp, sizeof(m_result));
+        tmp->queryData = tmp->cursor = NULL;
+        tmp->fieldData = tmp->fieldCursor = NULL;
+        tmp->numRows = numFields = 0;
+
 	tmp->queryData = queryData;
 	tmp->numRows = queryTableSize;
 	tmp->fieldData = tableToFieldList(fieldData);
@@ -861,111 +954,6 @@ void msqlDataSeek(handle, offset)
 
 
 
-/**************************************************************************
-**	_
-**
-**	Purpose	: 
-**	Args	: 
-**	Returns	: 
-**	Notes	: 
-*/
-
-int msqlQuery(sock,q)
-	int	sock;
-	char	*q;
-{
-	int	len,
-		res;
-	char	*cp;
-	
-
-
-        msqlDebug(MOD_QUERY,"Query = \"%s\"\n",q);
-	resetError();
-	setServerSock(sock);
-
-	/*
-	** Issue the query
-	*/
-
-	(void)sprintf(packet,"%d:%s\n",QUERY,q);
-	writePkt(sock);
-	(void)bzero(packet,PKT_LEN);
-	if(readPkt(sock) <= 0)
-	{
-		closeServer(sock);
-		strcpy(msqlErrMsg,SERVER_GONE_ERROR);
-		return(-1);
-	}
-
-
-	/*
-	** Check the result.  It may be an indication of further data to
-	** come (ie. from a select)
-	*/
-
-	if (atoi(packet) == -1)
-	{
-		cp = (char *)index(packet,':');
-		if (cp)
-		{
-			strcpy(msqlErrMsg,cp+1);
-			chopError();
-		}
-		else
-		{
-			strcpy(msqlErrMsg,UNKNOWN_ERROR);
-		}
-		return(-1);
-	}
-
-	cp = (char *)index(packet,':');
-	numFields = 0;
-	if (cp)
-	{
-		numFields = atoi(cp+1);
-		if (numFields <= 0)
-			return(0);
-	}
-	else
-	{
-		return(0);
-	}
-
-	/*
-	** numFields > 0 therefore we have data waiting on the socket.
-	** Grab it and dump it into a table structure.  If there's
-	** uncollected data free it - it's no longer available.
-	*/
-	if (queryData)
-	{
-		freeQueryData(queryData);
-		freeQueryData(fieldData);
-		queryData = NULL;
-		fieldData = NULL;
-	}
-
-	queryTableSize = readQueryData(sock);
-	if (queryTableSize < 0)
-	{
-		return(-1);
-	}
-	queryData = tmpDataStore;
-	tmpDataStore = NULL;
-	numFields = 6;
-	fieldTableSize = readQueryData(sock);
-	if (fieldTableSize < 0)
-	{
-		return(-1);
-	}
-	fieldData = tmpDataStore;
-	tmpDataStore = NULL;
-	return(0);
-}
-
-
-
-
 /**************************************************************************
 **	_
 **
@@ -981,7 +969,8 @@ int readQueryData(sock)
 	int	off,
 		len,
 		numRows;
-	char	*cp;
+	char	*cp,
+		*tmpcp;
 	m_data	*cur;
 	
 	if (readPkt(sock) <= 0)
@@ -1020,7 +1009,9 @@ int readQueryData(sock)
 		}
 		msqlDebug(MOD_MALLOC,"Query data row - malloc @ %X of %d\n",
 			cur, sizeof(m_data));
-		(void)bzero(cur,sizeof(m_data));
+		cur->width = 0;
+        	cur->next = NULL;
+
 		cur->data = (char **)malloc(numFields * sizeof(char *));
 		(void)bzero(cur->data,numFields * sizeof(char *));
 		cur->width = numFields;
@@ -1038,8 +1029,9 @@ int readQueryData(sock)
 			else
 			{
 				cur->data[off] = (char *)malloc(len+1);
-				(void)bzero(cur->data[off],len+1);
 				(void)bcopy(cp+1,cur->data[off],len);
+				tmpcp = (char *)cur->data[off];
+				*(tmpcp + len) = 0;
 				cp += len + 1;
 			}
 			off++;
@@ -1059,6 +1051,113 @@ int readQueryData(sock)
 
 
 
+/**************************************************************************
+**	_
+**
+**	Purpose	: 
+**	Args	: 
+**	Returns	: 
+**	Notes	: 
+*/
+
+int msqlQuery(sock,q)
+	int	sock;
+	char	*q;
+{
+	char	*cp;
+	int	res,
+		count = 0;
+	
+
+
+        msqlDebug(MOD_QUERY,"Query = \"%s\"\n",q);
+	resetError();
+	setServerSock(sock);
+
+	/*
+	** Issue the query
+	*/
+
+	(void)sprintf(packet,"%d:%s\n",QUERY,q);
+	writePkt(sock);
+	if(readPkt(sock) <= 0)
+	{
+		closeServer(sock);
+		strcpy(msqlErrMsg,SERVER_GONE_ERROR);
+		return(-1);
+	}
+
+
+	/*
+	** Check the result.  It may be an indication of further data to
+	** come (ie. from a select)
+	*/
+
+	res = atoi(packet);
+	if (res == -1)
+	{
+		cp = (char *)index(packet,':');
+		if (cp)
+		{
+			strncpy(msqlErrMsg,cp+1, ERR_BUF_LEN - 1);
+			*(msqlErrMsg + ERR_BUF_LEN - 1) = 0;
+			chopError();
+		}
+		else
+		{
+			strcpy(msqlErrMsg,UNKNOWN_ERROR);
+		}
+		return(-1);
+	}
+
+	cp = (char *)index(packet,':');
+	numFields = 0;
+	if (cp)
+	{
+		numFields = atoi(cp+1);
+		if (numFields <= 0)
+			return(res);
+	}
+	else
+	{
+		return(res);
+	}
+
+	/*
+	** numFields > 0 therefore we have data waiting on the socket.
+	** Grab it and dump it into a table structure.  If there's
+	** uncollected data free it - it's no longer available.
+	*/
+	if (queryData)
+	{
+		freeQueryData(queryData);
+		freeQueryData(fieldData);
+		queryData = NULL;
+		fieldData = NULL;
+	}
+
+	queryTableSize = readQueryData(sock);
+	if (queryTableSize < 0)
+	{
+		return(-1);
+	}
+	count = queryTableSize;
+	queryData = tmpDataStore;
+	tmpDataStore = NULL;
+	numFields = 6;
+	fieldTableSize = readQueryData(sock);
+	if (fieldTableSize < 0)
+	{
+		return(-1);
+	}
+	fieldData = tmpDataStore;
+	tmpDataStore = NULL;
+	return(count);
+}
+
+
+
+
 
 /**************************************************************************
 **	_
@@ -1209,18 +1308,125 @@ m_result *msqlListFields(sock,table)
 }
 
 
+/**************************************************************************
+**	_
+**
+**	Purpose	: 
+**	Args	: 
+**	Returns	: 
+**	Notes	: 
+*/
 
-
-int msqlCreateDB(sock,DB)
+m_seq *msqlGetSequenceInfo(sock,table)
 	int	sock;
-	char	*DB;
+	char	*table;
+{
+	static m_seq seq;
+	char	*cp;
+
+	msqlDebug(MOD_API,"msqlGetSequenceInfo(%d,%s)\n",sock,table);
+	sprintf(packet,"%d:%s\n",SEQ_INFO,table);
+	writePkt(sock);
+
+        if(readPkt(sock) <= 0)
+        {
+                closeServer(sock);
+                strcpy(msqlErrMsg,SERVER_GONE_ERROR);
+                return(NULL);
+        }
+
+
+        /*
+        ** Check the result.
+        */
+
+        if (atoi(packet) == -1)
+        {
+		cp = (char *)index(packet,':');
+		if (cp)
+		{
+			strcpy(msqlErrMsg,cp+1);
+			chopError();
+		}
+		else
+		{
+			strcpy(msqlErrMsg,UNKNOWN_ERROR);
+		}
+		return(NULL);
+        }
+        cp = (char *)index(packet,':');
+	seq.step = atoi(cp+1);
+	cp = (char *)index(cp+1,':');
+	seq.value = atoi(cp+1);
+	return(&seq);
+}
+
+
+
+/**************************************************************************
+**	_
+**
+**	Purpose	: 
+**	Args	: 
+**	Returns	: 
+**	Notes	: 
+*/
+
+m_result *msqlListIndex(sock,table, index)
+	int	sock;
+	char	*table,
+		*index;
+{
+	m_result *tmp;
+
+	msqlDebug(MOD_API,"msqlListIndex(%d,%s,%s)\n",sock,table,index);
+	tmp = (m_result *)malloc(sizeof(m_result));
+	if (!tmp)
+	{
+		return(NULL);
+	}
+	msqlDebug(MOD_MALLOC,"Result Handle - malloc @ %X of %d\n",
+		tmp, sizeof(m_result));
+	(void)bzero(tmp, sizeof(m_result));
+	sprintf(packet,"%d:%s:%s\n",INDEX_LIST,table,index);
+	writePkt(sock);
+	numFields = 1;
+	tmp->numFields = readQueryData(sock);
+	if(tmp->numFields < 0)
+	{
+		(void)free(tmp);
+		return(NULL);
+	}
+	tmp->queryData = tmpDataStore;
+	tmp->numFields = 0;
+	tmp->cursor = tmp->queryData;
+	tmp->fieldCursor = NULL;
+	tmpDataStore = NULL;
+	tmp->numFields = 1;
+	tmp->fieldData = (m_fdata *)malloc(sizeof(m_fdata));
+	msqlDebug(MOD_MALLOC,"Field List Entry - malloc @ %X of %d\n",
+		tmp->fieldData, sizeof(m_fdata));
+	(void)bzero(tmp->fieldData, sizeof(m_fdata));
+	tmp->fieldData->field.table = (char *)strdup("mSQL Catalog");
+	tmp->fieldData->field.name = (char *)strdup("Index");
+	tmp->fieldData->field.type = CHAR_TYPE;
+	tmp->fieldData->field.length = NAME_LEN;
+	tmp->fieldData->field.flags = 0;
+	return(tmp);
+}
+
+
+
+
+int msqlCreateDB(sock,db)
+	int	sock;
+	char	*db;
 {
 	char	*cp;
 
-	msqlDebug(MOD_API,"msqlCreateDB(%d,%s)\n",sock,DB);
-	sprintf(packet,"%d:%s\n",CREATE_DB,DB);
+	msqlDebug(MOD_API,"msqlCreateDB(%d,%s)\n",sock,db);
+	sprintf(packet,"%d:%s\n",CREATE_DB,db);
 	writePkt(sock);
-	(void)bzero(packet,PKT_LEN);
 	if(readPkt(sock) <= 0)
 	{
 		closeServer(sock);
@@ -1250,16 +1456,98 @@ int msqlCreateDB(sock,DB)
 	return(0);
 }
 
-int msqlDropDB(sock,DB)
+
+
+int msqlCopyDB(sock,fromDB, toDB)
 	int	sock;
-	char	*DB;
+	char	*fromDB,
+		*toDB;
 {
 	char	*cp;
 
-	msqlDebug(MOD_API,"msqlDropDB(%d,%s)\n",sock,DB);
-	sprintf(packet,"%d:%s\n",DROP_DB,DB);
+	msqlDebug(MOD_API,"msqlCopyDB(%d,%s,%s)\n",sock,fromDB, toDB);
+	sprintf(packet,"%d:%s:%s\n",COPY_DB,fromDB, toDB);
+	writePkt(sock);
+	if(readPkt(sock) <= 0)
+	{
+		closeServer(sock);
+		strcpy(msqlErrMsg,SERVER_GONE_ERROR);
+		return(-1);
+	}
+
+
+	/*
+	** Check the result.  
+	*/
+
+	if (atoi(packet) == -1)
+	{
+		cp = (char *)index(packet,':');
+		if (cp)
+		{
+			strcpy(msqlErrMsg,cp+1);
+			chopError();
+		}
+		else
+		{
+			strcpy(msqlErrMsg,UNKNOWN_ERROR);
+		}
+		return(-1);
+	}
+	return(0);
+}
+
+
+
+int msqlMoveDB(sock,fromDB, toDB)
+	int	sock;
+	char	*fromDB,
+		*toDB;
+{
+	char	*cp;
+
+	msqlDebug(MOD_API,"msqlMoveDB(%d,%s,%s)\n",sock,fromDB, toDB);
+	sprintf(packet,"%d:%s:%s\n",MOVE_DB,fromDB, toDB);
+	writePkt(sock);
+	if(readPkt(sock) <= 0)
+	{
+		closeServer(sock);
+		strcpy(msqlErrMsg,SERVER_GONE_ERROR);
+		return(-1);
+	}
+
+
+	/*
+	** Check the result.  
+	*/
+
+	if (atoi(packet) == -1)
+	{
+		cp = (char *)index(packet,':');
+		if (cp)
+		{
+			strcpy(msqlErrMsg,cp+1);
+			chopError();
+		}
+		else
+		{
+			strcpy(msqlErrMsg,UNKNOWN_ERROR);
+		}
+		return(-1);
+	}
+	return(0);
+}
+
+
+int msqlDropDB(sock,db)
+	int	sock;
+	char	*db;
+{
+	char	*cp;
+
+	msqlDebug(MOD_API,"msqlDropDB(%d,%s)\n",sock,db);
+	sprintf(packet,"%d:%s\n",DROP_DB,db);
 	writePkt(sock);
-	(void)bzero(packet,PKT_LEN);
 	if(readPkt(sock) <= 0)
 	{
 		closeServer(sock);
@@ -1298,7 +1586,6 @@ int msqlShutdown(sock)
 	msqlDebug(MOD_API,"msqlShutdown(%d)\n",sock);
 	sprintf(packet,"%d:\n",SHUTDOWN);
 	writePkt(sock);
-	(void)bzero(packet,PKT_LEN);
 	if(readPkt(sock) <= 0)
 	{
 		closeServer(sock);
@@ -1338,7 +1625,6 @@ int msqlReloadAcls(sock)
 	msqlDebug(MOD_API,"msqlReloadAcl(%d)\n",sock);
 	sprintf(packet,"%d:\n",RELOAD_ACL);
 	writePkt(sock);
-	(void)bzero(packet,PKT_LEN);
 	if(readPkt(sock) <= 0)
 	{
 		closeServer(sock);
@@ -1387,3 +1673,28 @@ int msqlGetProtoInfo()
 	return(protoInfo);
 }
 
+
+int msqlGetServerStats(sock)
+	int	sock;
+{
+	msqlDebug(MOD_API,"msqlShutdown(%d)\n",sock);
+	sprintf(packet,"%d:\n",SERVER_STATS);
+	writePkt(sock);
+	if(readPkt(sock) <= 0)
+	{
+		closeServer(sock);
+		strcpy(msqlErrMsg,SERVER_GONE_ERROR);
+		return(-1);
+	}
+	while(atoi(packet) != -100)
+	{
+		printf(packet);
+		if(readPkt(sock) <= 0)
+		{
+			closeServer(sock);
+			strcpy(msqlErrMsg,SERVER_GONE_ERROR);
+			return(-1);
+		}
+	}
+	return(0);
+}
