@@ -3,26 +3,11 @@
 **
 **
 ** Copyright (c) 1993-95  David J. Hughes
-** Copyright (c) 1995  Hughes Technologies Pty Ltd
-**
-** Permission to use, copy, and distribute for non-commercial purposes,
-** is hereby granted without fee, providing that the above copyright
-** notice appear in all copies and that both the copyright notice and this
-** permission notice appear in supporting documentation.
-**
-** This software is provided "as is" without any expressed or implied warranty.
-**
-** ID = "$Id:"
+** Copyright (c) 1995-96  Hughes Technologies Pty Ltd
 **
 */
 
 
-#ifndef lint
-static char RCS_id[] = 
-	"msql_proc.c,v 1.3 1994/08/19 08:03:15 bambi Exp";
-#endif 
-
-
 #include <stdio.h>
 #include <sys/types.h>
 
@@ -31,15 +16,30 @@ static char RCS_id[] =
 #include <arpa/inet.h>
 #include <signal.h>
 #include <netdb.h>
-
+#include <unistd.h>
+#include <stdlib.h>
+#include <sys/time.h>
+#include <string.h>
 
 #include <common/debug.h>
 #include <common/portability.h>
 
-#define	MSQL_ADT
+#ifdef WIN32
+#include <winsock.h>
+#endif
+
+
+
+#define MSQL_ADT
+#define _MSQL_SERVER_SOURCE
 #include "msql_priv.h"
 #include "msql.h"
-#include "y.tab.h"
+
+#if defined(WIN32)
+#  include "msql_yacc.h"
+#else
+#  include "y.tab.h"
+#endif
 
 int     command,
 	notnullflag,
@@ -51,13 +51,15 @@ char 	*curDB,
 	*arrayLen;
 
 cond_t	*condHead = NULL;
-
+cstk_t 	*condStack = NULL;
 field_t	*fieldHead = NULL,
 	*lastField = NULL;
-
 order_t	*orderHead = NULL;
-
 tname_t	*tableHead = NULL;
+mindex_t	indexHead;
+time_t	queryTime;
+char	seqTable[NAME_LEN + 1];
+seq_t	sequence;
 
 static	cond_t	*condTail = NULL;
 static	field_t	*fieldTail = NULL;
@@ -66,15 +68,39 @@ static	tname_t	*tableTail = NULL;
 static 	int	havePriKey = 0;
 
 
-#define	malloc(L)	Malloc(L,__FILE__,__LINE__)
-#define	free(A)		Free(A,__FILE__,__LINE__)
-#define	safeFree(x)	{if (x) { (void) free(x); x = NULL; }}
 #define REG		register
 
 
 extern	char	*packet;
 
 
+/*
+** A few things to reduce the amount of mallocing we do
+*/
+
+#define	MAX_SPARE_VALUES	15
+static 	val_t *spareValues[MAX_SPARE_VALUES];
+static	int numSpareValues = 0;
+
+
+
+static void cleanCond(curCond)
+	cond_t 	*curCond;
+{
+	cond_t *tmpCond;
+
+	while(curCond)
+	{
+		if (curCond->subCond != NULL)
+			cleanCond(curCond->subCond);
+		msqlFreeValue(curCond->value);
+		curCond->op = curCond->bool = 0;
+		tmpCond = curCond;
+		curCond = curCond->next;
+		(void)free(tmpCond);
+	}
+}
+
 
 /****************************************************************************
 ** 	_msqlClean - clean out the internal structures
@@ -85,16 +111,23 @@ extern	char	*packet;
 **	Notes	: Updates all public data structures
 */
 
-msqlClean()
+void msqlClean()
 {
 	register field_t	*curField, *tmpField;
-	register cond_t		*curCond, *tmpCond;
 	register order_t 	*curOrder, *tmpOrder;
 	register tname_t	*curTable, *tmpTable;
+	extern	u_char		*yyprev, *yytext;
 
 	msqlTrace(TRACE_IN,"msqlClean()");
+	if (yyprev)
+	{
+		free(yyprev);
+		yyprev = NULL;
+	}
+	yytext = NULL;
 	command = 0;
 	havePriKey = 0;
+	msqlSelectLimit = 0;
 
 	/*
 	** blow away the table list from the query
@@ -114,7 +147,7 @@ msqlClean()
 	curField = fieldHead;
 	while(curField)
 	{
-		freeValue(curField->value);
+		msqlFreeValue(curField->value);
 		tmpField = curField;
 		curField = curField->next;
 		(void)free(tmpField);
@@ -123,15 +156,8 @@ msqlClean()
 	/*
 	** Blow away the condition list from the query
 	*/
-	curCond = condHead;
-	while(curCond)
-	{
-		freeValue(curCond->value);
-		curCond->op = curCond->bool = 0;
-		tmpCond = curCond;
-		curCond = curCond->next;
-		(void)free(tmpCond);
-	}
+	cleanCond(condHead);
+
 	curOrder = orderHead;
 	while(curOrder)
 	{
@@ -152,6 +178,7 @@ msqlClean()
 	tableHead = tableTail = (tname_t *) NULL;
 
 	msqlBackendClean();
+	msqlResetSysVars();
 
 	msqlTrace(TRACE_OUT,"msqlClean()");
 }
@@ -167,7 +194,7 @@ ident_t *msqlCreateIdent(seg1,seg2)
 	msqlTrace(TRACE_IN,"msqlCreateIdent()");
 	if (seg1)
 	{
-		if (strlen(seg1) > NAME_LEN)
+		if ((int)strlen(seg1) > NAME_LEN)
 		{
 			sprintf(packet,
 				"-1:Identifier name '%s' too long\n",seg1);
@@ -178,7 +205,7 @@ ident_t *msqlCreateIdent(seg1,seg2)
 	}
 	if (seg2)
 	{
-		if (strlen(seg2) > NAME_LEN)
+		if ((int)strlen(seg2) > NAME_LEN)
 		{
 			sprintf(packet,
 				"-1:Identifier name '%s' too long\n",seg2);
@@ -187,15 +214,24 @@ ident_t *msqlCreateIdent(seg1,seg2)
 			return(NULL);
 		}
 	}
-	new = (ident_t *)malloc(sizeof(ident_t));
+	new = (ident_t *)fastMalloc(sizeof(ident_t));
+	*(new->seg1) = *(new->seg2) = 0;
 	if (seg1)
 	{
 		(void)strcpy(new->seg1,seg1);
 	}
+	else
+	{
+		*(new->seg1) = 0;
+	}
 	if (seg2)
 	{
 		(void)strcpy(new->seg2,seg2);
 	}
+	else
+	{
+		*(new->seg1) = 0;
+	}
 	msqlTrace(TRACE_OUT,"msqlCreateIdent()");
 	return(new);
 }
@@ -238,27 +274,35 @@ val_t *msqlCreateValue(textRep,type,tokLen)
 {
 	val_t	*new;
 	int	length,
-		remain,
-		match;
+		remain;
 	REG 	u_char	*cp,
 			*cp2;
-	static	u_char nullVal[] = "null";
+	char	*cp3;
 
 	msqlTrace(TRACE_IN,"msqlCreateValue()");
-	new = (val_t *)malloc(sizeof(val_t));
-	*(new->table) = *(new->name) = 0;
-	new->offs=0;
+	if (numSpareValues > 0)
+	{
+                numSpareValues--;
+                new = spareValues[numSpareValues];
+	}
+	else
+	{
+		new = (val_t *)malloc(sizeof(val_t));
+	}
+
 	new->type = type;
 	new->dataLen = tokLen;
+	new->nullVal = 0;
+	new->precision = 0;
 	switch(type)
 	{
 		case NULL_TYPE:
 			new->nullVal = 1;
 			break;
 		case IDENT_TYPE:
+		case SYSVAR_TYPE:
 			new->val.identVal = (ident_t *)textRep;
 			break;
-
 		case CHAR_TYPE:
 			remain = length = tokLen - 2;
 			new->val.charVal = (u_char *)malloc(length+1);
@@ -286,11 +330,20 @@ val_t *msqlCreateValue(textRep,type,tokLen)
 			break;
 
 		case INT_TYPE:
-			new->val.intVal = atoi(textRep);
+		case UINT_TYPE:
+			new->val.intVal = atoi((char *)textRep);
 			break;
 
 		case REAL_TYPE:
 			sscanf((char *)textRep ,"%lg",&new->val.realVal);
+			cp3 = (char *)index(textRep,'.');
+			if (cp3)
+			{
+				new->precision = strlen(textRep) -
+					(cp3 - (char *)textRep) - 1;
+				if (new->precision < 0)
+					new->precision = 0;
+			}
 			break;
 	}
 	msqlTrace(TRACE_OUT,"msqlCreateValue()");
@@ -306,7 +359,16 @@ val_t *fillValue(val,type,length)
 	val_t	*new;
 
 	msqlTrace(TRACE_IN,"fillValue()");
-	new = (val_t *)malloc(sizeof(val_t));
+        if (numSpareValues > 0)
+        {
+                numSpareValues--;
+                new = spareValues[numSpareValues];
+        }
+        else
+        {
+                new = (val_t *)malloc(sizeof(val_t));
+        }
+
 	new->type = type;
 	new->nullVal = 0;
 	switch(type)
@@ -317,6 +379,10 @@ val_t *fillValue(val,type,length)
 			break;
 
 		case INT_TYPE:
+		case UINT_TYPE:
+		case DATE_TYPE:
+		case MONEY_TYPE:
+		case TIME_TYPE:
 #ifndef _CRAY
 			new->val.intVal = (int) * (int *)val;
 #else
@@ -337,38 +403,123 @@ val_t *nullValue()
 {
 	val_t	*new;
 
-	new = (val_t *)malloc(sizeof(val_t));
+        if (numSpareValues > 0)
+        {
+                numSpareValues--;
+                new = spareValues[numSpareValues];
+        }
+        else
+        {
+                new = (val_t *)malloc(sizeof(val_t));
+        }
+
 	new->nullVal = 1;
 	return(new);
 }
 
 
 
-freeValue(val)
+void msqlFreeValue(val)
 	val_t	*val;
 {
-	msqlTrace(TRACE_IN,"freeValue()");
+	msqlTrace(TRACE_IN,"msqlFreeValue()");
 	if (!val)
 	{
-		msqlTrace(TRACE_OUT,"freeValue()");
+		msqlTrace(TRACE_OUT,"msqlFreeValue()");
 		return;
 	}
-	switch(val->type)
+	if (!val->nullVal)
 	{
-		case IDENT_TYPE:
-			(void)free(val->val.identVal);
-			break;
-		case CHAR_TYPE:
-			if (!val->nullVal)
-				(void)free(val->val.charVal);
-			break;
+		switch(val->type)
+		{
+			case IDENT_TYPE:
+			case SYSVAR_TYPE:
+				if (val->val.identVal)
+					(void)free(val->val.identVal);
+				val->val.identVal = NULL;
+				break;
+			case CHAR_TYPE:
+			case TEXT_TYPE:
+				if (val->val.charVal)
+					(void)free(val->val.charVal);
+				val->val.charVal = NULL;
+				break;
+		}
 	}
-	(void)free(val);
-	msqlTrace(TRACE_OUT,"freeValue()");
+	if (numSpareValues < MAX_SPARE_VALUES)
+	{
+		spareValues[numSpareValues] = val;
+		numSpareValues++;
+	}
+	else
+	{
+		(void)free(val);
+	}
+	msqlTrace(TRACE_OUT,"msqlFreeValue()");
 }
 
 
 
+
+
+void msqlAddSequence(table, step, val)
+	char	*table;
+	int	step,
+		val;
+{
+	msqlTrace(TRACE_IN,"msqlAddSequence()");
+
+	strcpy(seqTable,table);
+	sequence.step = step;
+	sequence.value = val;
+
+	msqlTrace(TRACE_OUT,"msqlAddSequence()");
+}
+
+
+
+
+int msqlAddSetFunct(funct, ident)
+	char	*funct;
+	ident_t	*ident;
+{
+	register field_t	*new;
+	char	*name,
+		*table;
+
+	msqlTrace(TRACE_IN,"msqlAddSetField()");
+
+	name = ident->seg2;
+	table = ident->seg1;
+
+	if (checkSetFunctName(funct) < 0)
+	{
+		msqlTrace(TRACE_OUT,"msqlAddSetFunct()");
+		return(-1);
+	}
+	new = (field_t *)malloc(sizeof(field_t));
+	if (table)
+	{
+		(void)strncpy(new->table,table,NAME_LEN - 1);
+	}
+	(void)strncpy(new->name,name,NAME_LEN - 1);
+	(void)strncpy(new->funct,funct,NAME_LEN - 1);
+
+	if (!fieldHead)
+	{
+		fieldHead = fieldTail = new;
+	}
+	else
+	{
+		fieldTail->next = new;
+		fieldTail = new;
+	}
+	free(ident);
+	msqlTrace(TRACE_OUT,"msqlAddSetField()");
+	return(0);
+}
+
+
 /****************************************************************************
 ** 	_msqlAddField - add a field definition to the list
 **
@@ -421,73 +572,67 @@ int msqlAddField(ident,type,length,notNull,priKey)
 
 	if (priKey)
 	{
-		if (havePriKey)
-		{
-			sprintf(packet,"-1:Can't have multiple primary keys\n");
-			writePkt(outSock);
-			msqlTrace(TRACE_OUT,"msqlAddField()");
-			return(-1);
-
-		}
-		else
-		{
-			havePriKey = 1;
-		}
-	}
-
-
-	new = (field_t *)malloc(sizeof(field_t));
-	if (table)
-	{
-		if (strlen(table) > NAME_LEN)
-		{
-			sprintf(packet, "-1:Table name '%s' is too long\n",
-				table);
-			writePkt(outSock);
-			msqlTrace(TRACE_OUT,"msqlAddField()");
-			return(-1);
-		}
-		(void)strcpy(new->table,table);
-	}
-	if (strlen(name) > NAME_LEN)
-	{
-		sprintf(packet, "-1:Field name '%s' is too long\n", name);
+		sprintf(packet,
+			"-1:Primary keys are obsolete.  Use CREATE INDEX\n");
 		writePkt(outSock);
 		msqlTrace(TRACE_OUT,"msqlAddField()");
 		return(-1);
 	}
-	(void)strcpy(new->name,name);
+
+
+	new = (field_t *)fastMalloc(sizeof(field_t));
+	*(new->table) = *(new->name) = *(new->funct) = 0;
+	new->value = NULL;
+	new->entry = NULL;
+	new->next = NULL;
+	new->type = new->sysvar = new->length = new->dataLength =
+		new->offset = new->null = new->flags = 
+		new->fieldID = new->overflow = 0;
+
+
+	if (table)
+	{
+		(void)strncpy(new->table,table,NAME_LEN - 1);
+	}
+	(void)strncpy(new->name,name,NAME_LEN - 1);
 	if (notNull)
 	{
 		new->flags |= NOT_NULL_FLAG;
 	}
-	if (priKey)
-	{
-		new->flags |= PRI_KEY_FLAG;
-		new->flags |= NOT_NULL_FLAG;
-	}
 	switch(type)
 	{
-		case INT:
-			new->type = INT_TYPE;
-			new->length = 4;
+		case INT_TYPE:
+		case UINT_TYPE:
+		case DATE_TYPE:
+		case TIME_TYPE:
+		case MONEY_TYPE:
+			new->type = type;
+			new->dataLength = new->length = 4;
 			break;
 
-		case CHAR:
+		case CHAR_TYPE:
 			new->type = CHAR_TYPE;
-			new->length = atoi(length);
+			new->dataLength = new->length = atoi(length);
 			break;
 
-		case REAL:
+		case REAL_TYPE:
 			new->type = REAL_TYPE;
 			new->length = sizeof(double);
+			new->dataLength = new->length + 1;
+			break;
+
+		case TEXT_TYPE:
+			new->type = TEXT_TYPE;
+			new->length = atoi(length);
+			new->dataLength = new->length + VC_HEAD_SIZE;
 			break;
 
 		default:
 			new->type = 0;
-			new->length = 0;
+			new->dataLength = new->length = 0;
 			break;
 	}
+	new->overflow = NO_POS;
 	if (!fieldHead)
 	{
 		fieldHead = fieldTail = new;
@@ -502,20 +647,19 @@ int msqlAddField(ident,type,length,notNull,priKey)
 	return(0);
 }
 
-msqlSetSelectLimit(value)
+void msqlSetSelectLimit(value)
 	val_t	*value;
 {
 	msqlSelectLimit = value->val.intVal;
 }
 
 
-msqlAddFieldValue(value,ident,operator)
+void msqlAddFieldValue(value)
 	val_t	*value;
-	ident_t *ident;
-	int operator;
 {
 	register field_t	*fieldVal;
 	u_char	*buf;
+
 	msqlTrace(TRACE_IN,"msqlAddFieldValue()");
 	if (!lastField)
 	{
@@ -526,22 +670,6 @@ msqlAddFieldValue(value,ident,operator)
 		fieldVal = lastField->next;
 		lastField = lastField->next;
 	}
-	if (ident && operator)
-	{
-		value->op=operator;
-
-		if (*(ident->seg1))
-		{
-			(void)strncpy(value->table,ident->seg1,NAME_LEN - 1);
-			(void)strncpy(value->name ,ident->seg2,NAME_LEN - 1);
-		}
-		else
-		{
-			(void)memset(value->table,0,NAME_LEN);
-			(void)strncpy(value->name ,ident->seg2,NAME_LEN - 1);
-		}
-		free(ident);
-	}
 	if (fieldVal)
 	{
 		if (fieldVal->type == CHAR_TYPE)
@@ -551,8 +679,6 @@ msqlAddFieldValue(value,ident,operator)
 			free(value->val.charVal);
 			value->val.charVal = buf;
 		}
-		if (fieldVal->value)
-			freeValue(fieldVal->value);
 		fieldVal->value = value;
 	}
 	msqlTrace(TRACE_OUT,"msqlAddFieldValue()");
@@ -561,6 +687,58 @@ msqlAddFieldValue(value,ident,operator)
 
 
 
+void msqlPushCond()
+{
+	cstk_t	*new;
+
+	new = (cstk_t *)malloc(sizeof(cstk_t));
+	new->next = condStack;
+	new->head = condHead;
+	new->tail = condTail;
+	condStack = new;
+	condHead = condTail = NULL;
+}
+
+
+void msqlPopCond()
+{
+	cstk_t	*tmp;
+
+	tmp = condStack;
+	condStack = condStack->next;
+	condHead = tmp->head;
+	condTail = tmp->tail;
+	free(tmp);
+}
+
+
+void msqlAddSubCond(bool)
+	int	bool;
+{
+	register cond_t	*new;
+
+	msqlTrace(TRACE_IN,"msqlAddSubCond()");
+
+	new = (cond_t *)malloc(sizeof(cond_t));
+	new->op = 0;
+	new->bool = bool;
+	new->value = NULL;
+	new->subCond = condHead;
+	msqlPopCond();
+
+	if (!condHead)
+	{
+		condHead = condTail = new;
+	}
+	else
+	{
+		condTail->next = new;
+		condTail = new;
+	}
+	msqlTrace(TRACE_OUT,"msqlAddSubCond()");
+}
+
+
 /****************************************************************************
 ** 	_msqlAddCond  -  add a conditional spec to the list
 **
@@ -571,7 +749,7 @@ msqlAddFieldValue(value,ident,operator)
 **		  element of a where_clause.
 */
 
-int msqlAddCond(ident,op,value,bool)
+void msqlAddCond(ident,op,value,bool)
 	ident_t	*ident;
 	int	op;
 	val_t	*value;
@@ -583,36 +761,17 @@ int msqlAddCond(ident,op,value,bool)
 
 	msqlTrace(TRACE_IN,"msqlAddCond()");
 
-	if (*(ident->seg1))
+	if (*(ident->seg2))
 	{
 		name = ident->seg2;
 		table = ident->seg1;
 	}
 	else
 	{
-		name = ident->seg2;
+		name = ident->seg1;
 		table = NULL;
 	}
 
-	if (strlen(name) > NAME_LEN)
-	{
-		sprintf(packet, "-1:Field name '%s' is too long\n", 
-			name);
-		writePkt(outSock);
-		msqlTrace(TRACE_OUT,"msqlAddCond()");
-		return(-1);
-	}
-	if (table)
-	{
-		if (strlen(table) > NAME_LEN)
-		{
-			sprintf(packet, "-1:Table name '%s' is too long\n", 
-				name);
-			writePkt(outSock);
-			msqlTrace(TRACE_OUT,"msqlAddCond()");
-			return(-1);
-		}
-	}
 	new = (cond_t *)malloc(sizeof(cond_t));
 	(void)strcpy(new->name,name);
 	if (table)
@@ -623,6 +782,7 @@ int msqlAddCond(ident,op,value,bool)
 	new->op = op;
 	new->bool = bool;
 	new->value = value;
+	new->subCond = NULL;
 
 	if (!condHead)
 	{
@@ -635,7 +795,6 @@ int msqlAddCond(ident,op,value,bool)
 	}
 	free(ident);
 	msqlTrace(TRACE_OUT,"msqlAddCond()");
-	return(0);
 }
 
 
@@ -649,7 +808,7 @@ int msqlAddCond(ident,op,value,bool)
 **	Notes	: 
 */
 
-msqlAddOrder(ident,dir)
+void msqlAddOrder(ident,dir)
 	ident_t	*ident;
 	int	dir;
 {
@@ -680,86 +839,26 @@ msqlAddOrder(ident,dir)
 
 
 
-int msqlAddTable(name,alias)
+void msqlAddTable(name,alias)
 	char	*name,
 		*alias;
 {
-	register tname_t	*new,
-				*cur;
+	register tname_t	*new;
 
 	msqlTrace(TRACE_IN,"msqlAddTable()");
 
-	new = (tname_t *)malloc(sizeof(tname_t));
+	new = (tname_t *)fastMalloc(sizeof(tname_t));
+	*(new->name) = *(new->cname) = 0;
+	new->done = 0;
+	new->next = NULL;
+
 	if (alias)
 	{
-		/*
-		** Ensure the alias is unique
-		*/
-		cur = tableHead;
-		while(cur)
-		{
-			if (strcmp(cur->name,alias) == 0)
-			{
-				sprintf(packet, 
-					"-1:Nonunique table/alias name '%s'\n", 
-					alias);
-				writePkt(outSock);
-				msqlTrace(TRACE_OUT,"msqlAddTable()");
-				return(-1);
-			}
-			cur = cur->next;
-		}
-
-		/*
-		** Further checks on the table/alias name
-		*/
-		if (strlen(name) > NAME_LEN)
-		{
-			sprintf(packet, "-1:Table name '%s' is too long\n", 
-				name);
-			writePkt(outSock);
-			msqlTrace(TRACE_OUT,"msqlAddTable()");
-			return(-1);
-		}
-		if (strlen(alias) > NAME_LEN)
-		{
-			sprintf(packet,"-1:Table alias name '%s' is too long\n",
-				name);
-			writePkt(outSock);
-			msqlTrace(TRACE_OUT,"msqlAddTable()");
-			return(-1);
-		}
 		(void)strcpy(new->name,alias);
 		(void)strcpy(new->cname,name);
 	}
 	else
 	{
-		/*
-		** Ensure the table is unique
-		*/
-		cur = tableHead;
-		while(cur)
-		{
-			if (strcmp(cur->name,name) == 0)
-			{
-				sprintf(packet, 
-					"-1:Nonunique table name '%s'\n", 
-					name);
-				writePkt(outSock);
-				msqlTrace(TRACE_OUT,"msqlAddTable()");
-				return(-1);
-			}
-			cur = cur->next;
-		}
-
-		if (strlen(name) > NAME_LEN)
-		{
-			sprintf(packet, "-1:Table name '%s' is too long\n", 
-				name);
-			writePkt(outSock);
-			msqlTrace(TRACE_OUT,"msqlAddTable()");
-			return(-1);
-		}
 		(void)strcpy(new->name,name);
 		*(new->cname) = 0;
 	}
@@ -773,16 +872,26 @@ int msqlAddTable(name,alias)
 		tableTail = new;
 	}
 	msqlTrace(TRACE_OUT,"msqlAddTable()");
-	return(0);
 }
 
 
 
+void msqlAddIndex(name, table, uniq, type)
+	char	*name,
+		*table;
+	int	uniq,
+		type;
+{
+	msqlTrace(TRACE_IN,"msqlAddIndex()");
+	strncpy(indexHead.name,name,NAME_LEN);
+	strncpy(indexHead.table,table,NAME_LEN);
+	indexHead.unique = uniq;
+	indexHead.idxType = type;
+	msqlTrace(TRACE_OUT,"msqlAddIndex()");
+}
 
 
-
-
-msqlSetDB(db)
+void msqlSetDB(db)
 	char	*db;
 {
 	curDB = db;
@@ -803,9 +912,10 @@ msqlSetDB(db)
 
 
 
-msqlProcessQuery()
+void msqlProcessQuery()
 {
 	int	res;
+	extern	int readOnly;
 
 	msqlTrace(TRACE_IN,"msqlProcessQuery()");
 	if (!curDB)
@@ -816,9 +926,10 @@ msqlProcessQuery()
 		msqlTrace(TRACE_OUT,"msqlProcessQuery()");
 		return;
 	}
+	queryTime = time(NULL);
 	switch(command)
 	{
-		case SELECT: 
+		case MSQL_SELECT: 
 			if (!msqlCheckPerms(READ_ACCESS))
 			{
 				sprintf(packet,"-1:Access Denied\n");
@@ -826,65 +937,106 @@ msqlProcessQuery()
 				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
 				return;
 			}
-			res = msqlSelect(tableHead,fieldHead,condHead,
+			res = msqlServerSelect(tableHead,fieldHead,condHead,
 				orderHead,curDB);
 			break;
-		case CREATE: 
-			if (!msqlCheckPerms(WRITE_ACCESS))
+		case CREATE_TABLE: 
+			if (!msqlCheckPerms(WRITE_ACCESS) || readOnly)
 			{
 				sprintf(packet,"-1:Access Denied\n");
 				writePkt(outSock);
 				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
 				return;
 			}
-			res = msqlCreate(tableHead->name,fieldHead,curDB);
-			break;
-		case UPDATE: 
-			if (!msqlCheckPerms(RW_ACCESS))
-			{
-				sprintf(packet,"-1:Access Denied\n");
-				writePkt(outSock);
-				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
-				return;
-			}
-			res = msqlUpdate(tableHead->name,fieldHead,condHead,
+			res = msqlServerCreateTable(tableHead->name,fieldHead,
 				curDB);
 			break;
-		case INSERT: 
-			if (!msqlCheckPerms(WRITE_ACCESS))
+		case CREATE_INDEX: 
+			if (!msqlCheckPerms(WRITE_ACCESS) || readOnly)
 			{
 				sprintf(packet,"-1:Access Denied\n");
 				writePkt(outSock);
 				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
 				return;
 			}
-			res = msqlInsert(tableHead->name,fieldHead,curDB);
+			res = msqlServerCreateIndex(&indexHead,fieldHead,curDB);
 			break;
-		case DELETE: 
-			if (!msqlCheckPerms(WRITE_ACCESS))
+		case CREATE_SEQUENCE: 
+			if (!msqlCheckPerms(WRITE_ACCESS) || readOnly)
 			{
 				sprintf(packet,"-1:Access Denied\n");
 				writePkt(outSock);
 				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
 				return;
 			}
-			res = msqlDelete(tableHead->name,condHead,curDB);
+			res = msqlServerCreateSequence(seqTable,sequence.step,
+				sequence.value,curDB);
 			break;
-		case DROP: 
-			if (!msqlCheckPerms(WRITE_ACCESS))
+		case MSQL_UPDATE: 
+			if (!msqlCheckPerms(RW_ACCESS) || readOnly)
 			{
 				sprintf(packet,"-1:Access Denied\n");
 				writePkt(outSock);
 				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
 				return;
 			}
-			res = msqlDrop(tableHead->name,curDB);
+			res = msqlServerUpdate(tableHead->name,fieldHead,
+				condHead, curDB);
+			break;
+		case MSQL_INSERT: 
+			if (!msqlCheckPerms(WRITE_ACCESS) || readOnly)
+			{
+				sprintf(packet,"-1:Access Denied\n");
+				writePkt(outSock);
+				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
+				return;
+			}
+			res = msqlServerInsert(tableHead->name,fieldHead,curDB);
+			break;
+		case MSQL_DELETE: 
+			if (!msqlCheckPerms(WRITE_ACCESS) || readOnly)
+			{
+				sprintf(packet,"-1:Access Denied\n");
+				writePkt(outSock);
+				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
+				return;
+			}
+			res = msqlServerDelete(tableHead->name,condHead,curDB);
+			break;
+		case DROP_TABLE: 
+			if (!msqlCheckPerms(WRITE_ACCESS) || readOnly)
+			{
+				sprintf(packet,"-1:Access Denied\n");
+				writePkt(outSock);
+				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
+				return;
+			}
+			res = msqlServerDropTable(tableHead->name,curDB);
+			break;
+		case DROP_INDEX: 
+			if (!msqlCheckPerms(WRITE_ACCESS) || readOnly)
+			{
+				sprintf(packet,"-1:Access Denied\n");
+				writePkt(outSock);
+				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
+				return;
+			}
+			res = msqlServerDropIndex(&indexHead,curDB);
+			break;
+		case DROP_SEQUENCE: 
+			if (!msqlCheckPerms(WRITE_ACCESS) || readOnly)
+			{
+				sprintf(packet,"-1:Access Denied\n");
+				writePkt(outSock);
+				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
+				return;
+			}
+			res = msqlServerDropSequence(seqTable,curDB);
 			break;
 	}
 	if (res < 0)
 	{
 		extern	char	errMsg[];
-		char	errBuf[180];
 
 		sprintf(packet,"-1:%s\n",errMsg);
 		msqlTrace(TRACE_OUT,"msqlProcessQuery()");
@@ -894,7 +1046,7 @@ msqlProcessQuery()
 }
 
 
-msqlQueryOverrunError(txt)
+void msqlQueryOverrunError(txt)
 	char	*txt;
 {
 
@@ -907,13 +1059,14 @@ msqlQueryOverrunError(txt)
 
 
 
-char *msqlParseQuery(inBuf,sock)
+void msqlParseQuery(inBuf,sock)
         char    *inBuf;
         int     sock;
 {
 	msqlTrace(TRACE_IN,"msqlParseQuery()");
         outSock = sock;
-        msqlInitScanner(inBuf);
+        msqlInitScanner((u_char *)inBuf);
+	bzero(&indexHead, sizeof(indexHead));
         yyparse();
 	msqlTrace(TRACE_OUT,"msqlParseQuery()");
 }
