@@ -27,11 +27,23 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <stdlib.h>
-#include <varargs.h>
+#include <unistd.h>
+
+#ifdef WIN32
+#  include <winsock.h>
+#endif
+
+#ifndef WIN32
+#  include <pwd.h>
+#endif
+
+#include <string.h>
 
 #include "version.h"
 
-#include "common/portability.h"
+#ifndef WIN32
+#  include <common/config.h>
+#endif
 
 #ifdef HAVE_SYS_UN_H
 #  include <sys/un.h>
@@ -48,17 +60,34 @@
 #  include <sys/select.h>
 #endif
 
+#ifdef HAVE_SETRLIMIT
+#  include <sys/resource.h>
+#endif
+
 #include "common/site.h"
 #include "common/debug.h"
 
+#include "common/portability.h"
+
+#define	_MSQL_SERVER_SOURCE
 #include "msql_priv.h"
 #include "errmsg.h"
 
 
-char *timestamp();
+#ifdef ANSI_ARGS
+#  include <stdarg.h>
+#else
+#  include <varargs.h>
+#endif
 
 
-extern  char    *yytext;
+
+
+
+
+extern  u_char    *yytext,
+		*yyprev;
+extern  char    *msqlHomeDir;
 extern	int	yylineno;
 extern	int	yydebug;
 
@@ -67,31 +96,53 @@ extern	char	errMsg[];
 int	curSock,
 	IPsock,
 	UNIXsock,
-	numCons;
+	numCons,
+	maxCons,
+	msyncTimer,
+	startTime,
+	readOnly;
 
-char	*unixPort;
+char	*unixPort,
+	confFile[MAXPATHLEN];
 
-#define MAX_SOCK_NUM	100
 
-cinfo_t	conArray[MAX_SOCK_NUM];
+cinfo_t	conArray[256];
 
 extern	char	*packet;
 char	PROGNAME[] = "msqld",
 	BLANK_ARGV[] = "                                                  ";
 
-#define safeFree(x)  { if(x){ (void)free(x); x = NULL; } }
 
 
-void sigTrap(sig)
+static char *comTable[] = {
+        "???", "Quit", "Init DB", "Query", "DB List", "Table List",
+        "Field List", "Create DB", "Drop DB", "Reload ACL",
+        "Shutdown", "Index List", "Stats", "Seq Info", "Move DB",
+	"Copy DB", "???" };
+
+
+#ifdef WIN32
+RETSIGTYPE sigTrap(int sig);
+RETSIGTYPE puntServer(int sig);
+RETSIGTYPE puntClient(int sig);
+#endif
+
+
+
+RETSIGTYPE sigTrap(sig)
 	int	sig;
 {
 	int	clientSock;
 
+#ifdef WIN32
+	fprintf(stderr,"\nHit by signal\n\n");
+#else
 	signal(sig,SIG_IGN);
 	fprintf(stderr,"\nHit by a sig %d\n\n",sig);
+#endif
 	clientSock = 3;
 	printf("\n\nForced server shutdown due to bad signal!\n\n");
-	while(clientSock < MAX_SOCK_NUM)
+	while(clientSock < maxCons + CACHE_FDS)
 	{
 		if (conArray[clientSock].db)
 		{
@@ -107,13 +158,66 @@ void sigTrap(sig)
 	shutdown(UNIXsock,2);
 	close(UNIXsock);
 	unlink(unixPort);
+	free(unixPort);
 #endif
 	printf("\n");
 	abort();
 }
 
 
-sendError(fd,err)
+
+void mmapSyncer()
+{
+
+
+#ifdef HAVE_MSYNC
+	{
+		extern	cache_t tableCache[CACHE_SIZE];
+		cache_t	*cur;
+		int	count;
+		mindex_t *curIdx;
+
+
+		for (count = 0; count < CACHE_SIZE; count++)
+		{
+			cur = tableCache + count;
+			if (cur->age == 0)
+			{
+				count++;
+				continue;
+			}
+			if (cur->size)
+				msync(cur->dataMap,cur->size);
+			if (cur->overflowSize)
+				msync(cur->overflowMap,cur->overflowSize);
+			curIdx = cur->indices;
+			while(curIdx)
+			{
+				if (curIdx->idxType == IDX_AVL)
+				{
+					if(curIdx->tree->mapLen)
+						msync(curIdx->tree->mapRegion,
+					  		curIdx->tree->mapLen);
+				}
+				curIdx = curIdx->next;
+			}
+		}
+	}
+#endif
+
+#ifndef WIN32
+	sync();
+#endif
+
+#if !defined(WIN32) && !defined(OS2)
+	signal(SIGALRM, mmapSyncer);
+	alarm(msyncTimer);
+#endif
+}
+
+
+
+void sendError(fd,err)
 	char	*err;
 {
 #	ifdef DEBUG
@@ -126,7 +230,7 @@ sendError(fd,err)
 	writePkt(fd);
 }
 
-sendOK(fd)
+void sendOK(fd)
 {
 #	ifdef DEBUG
 		printf("Send OK called\n");
@@ -145,15 +249,11 @@ sendOK(fd)
 **	Notes	: 
 */
 
-initServer()
+void initServer()
 {
-	int	sock,
-		tcpPort,
-		addrLen,
+	int	tcpPort,
 		opt;
 	struct	sockaddr_in	IPaddr;
-	struct	servent		*serv_ptr;
-	char	*envVar;
 
 #ifdef HAVE_SYS_UN_H
 	struct	sockaddr_un	UNIXaddr;
@@ -162,15 +262,7 @@ initServer()
 	/*
 	** Create an IP socket
 	*/
-	tcpPort = MSQL_PORT;
-	if ((serv_ptr = getservbyname("msql", "tcp")))
-	{
-		tcpPort = serv_ptr->s_port;
-	}
-	if ((envVar = getenv("MSQL_TCP_PORT")))
-	{
-		tcpPort = atoi(envVar);
-	}
+	tcpPort = msqlGetIntConf("general", "tcp_port");
 	msqlDebug(MOD_GENERAL,"IP Socket is %d\n",tcpPort);
 	IPsock = socket(AF_INET, SOCK_STREAM, 0);
 	if (IPsock < 0)
@@ -180,7 +272,7 @@ initServer()
 	}
 #ifdef SO_REUSEADDR
 	opt = 1;
-	setsockopt(IPsock, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(int));
+	setsockopt(IPsock, SOL_SOCKET, SO_REUSEADDR, (char *)&opt,sizeof(int));
 #endif
 
 	(void)bzero(&IPaddr, sizeof(IPaddr));
@@ -192,17 +284,14 @@ initServer()
 		perror("Can't start server : IP Bind ");
 		exit(1);
 	}
-	listen(IPsock,5);
+	listen(IPsock,128);
 
 #ifdef HAVE_SYS_UN_H
 	/*
 	** Create the UNIX socket
 	*/
-	unixPort = MSQL_UNIX_ADDR;
-	if ((envVar = getenv("MSQL_UNIX_PORT")))
-	{
-		unixPort = envVar;
-	}
+	unixPort = (char *)strdup((char *)msqlGetCharConf("general", 
+		"unix_port"));
 	
 	msqlDebug(MOD_GENERAL,"UNIX Socket is %s\n",unixPort);
 	UNIXsock = socket(AF_UNIX, SOCK_STREAM, 0);
@@ -220,7 +309,8 @@ initServer()
 		perror("Can't start server : UNIX Bind ");
 		exit(1);
 	}
-	listen(UNIXsock,5);
+	listen(UNIXsock,128);
+	chmod(unixPort,0777);
 #endif
 }
 
@@ -241,7 +331,7 @@ RETSIGTYPE puntServer(sig)
 	{
 		printf("\n\nServer Aborting!\n\n");
 	}
-	while(clientSock < MAX_SOCK_NUM)
+	while(clientSock < maxCons + CACHE_FDS)
 	{
 		if (conArray[clientSock].db)
 		{
@@ -258,9 +348,10 @@ RETSIGTYPE puntServer(sig)
 	shutdown(UNIXsock,2);
 	close(UNIXsock);
 	unlink(unixPort);
+	free(unixPort);
 #endif
 	printf("\n");
-	dropCache();
+	msqlDropCache();
 
 	if (debugSet(MOD_MALLOC))
 	{
@@ -277,14 +368,17 @@ RETSIGTYPE puntServer(sig)
 	{
 		exit(1);
 	}
+	return;
 }
 
 
 
-yyerror(s)
+void yyerror(s)
         char	*s;
 {
- 	sprintf(packet,"-1:%s near \"%s\"\n", s, yytext?yytext:"");
+ 	sprintf(packet,"-1:%s at line %d near \"%s %s\"\n", s, 
+		yylineno, (char *)yyprev != NULL ?(char *)yyprev:"", 
+		(char *)yytext != NULL ?(char *)yytext:"");
 	writePkt(curSock);
 	msqlClean();
 }
@@ -296,7 +390,7 @@ yyerror(s)
 static int curComSock;
 static fd_set *clientFdSet;
 
-static setConnectionState(sock, fds)
+static void setConnectionState(sock, fds)
 	int	sock;
 	fd_set	*fds;
 {
@@ -305,6 +399,12 @@ static setConnectionState(sock, fds)
 }
 
 
+char *getCurUser()
+{
+	return(strdup(conArray[curComSock].user));
+}
+
+
 RETSIGTYPE puntClient(sig)
 	int	sig;
 {
@@ -321,15 +421,87 @@ RETSIGTYPE puntClient(sig)
 			safeFree(conArray[curComSock].user);
 		}
 		conArray[curComSock].db = NULL;
+		conArray[curComSock].user = NULL;
 		printf("Forced close of client on socket %d due to pipe sig\n",
 			curComSock);
 		numCons--;
 	}
+	return;
 }
 
 
 
-setupSignals()
+void sendServerStats(sock)
+	int	sock;
+{
+	struct	passwd *pw;
+	int	uid,
+		loop,
+		cTime;
+	u_int	curTime = time(NULL);
+	char	buf[10];
+
+	sprintf(packet,"Mini SQL Version %s\n",SERVER_VERSION);
+	strcat(packet,"Copyright (c) 1993-94 David J. Hughes\n");
+	strcat(packet,"Copyright (c) 1995-97 Hughes Technologies Pty Ltd.\n");
+	strcat(packet,"All rights reserved.\n\n");
+	writePkt(sock);
+	sprintf(packet,"Config file      : %s\n",confFile);
+	writePkt(sock);
+	sprintf(packet,"Max connections  : %d\n",maxCons);
+	writePkt(sock);
+	sprintf(packet,"Cur connections  : %d\n",numCons);
+	writePkt(sock);
+
+#if !defined(WIN32) && !defined(OS2)
+	uid = getuid();
+	pw = getpwuid(uid);
+	if (pw != NULL)
+	{
+		sprintf(packet,"Running as user  : %s\n", pw->pw_name);
+	}
+	else
+	{
+		sprintf(packet,"Running as user  : UID %d\n", uid);
+	}
+	writePkt(sock);
+#endif
+
+	strcpy(packet, "\nConnection table :\n");
+	writePkt(sock);
+	loop = 0;
+	strcpy(packet,
+	    "  Sock    Username       Hostname        Database    Connect   Idle   Queries\n");
+	strcat(packet,
+	    " +-----+------------+-----------------+------------+---------+------+--------+\n");
+	writePkt(sock);
+	while(loop < maxCons)
+	{
+		if (conArray[loop].user)
+		{
+			cTime = (int)(curTime - conArray[loop].connectTime)/60;
+			sprintf(buf,"%2dH %2dM",cTime/60, cTime%60);
+			sprintf(packet,
+			    " | %3d | %-10s | %-15s | %-10s | %s | %4d | %6d |\n",
+			    loop, conArray[loop].user, 
+			    conArray[loop].host?conArray[loop].host:"UNIX Sock",
+			    conArray[loop].db ? conArray[loop].db : "No DB",
+			    buf,
+			    (int)(curTime - conArray[loop].lastQuery)/60,
+			    conArray[loop].numQueries);
+			writePkt(sock);
+		}
+		loop++;
+	}
+	strcpy(packet,
+	    " +-----+------------+-----------------+------------+---------+------+--------+\n");
+	writePkt(sock);
+
+	strcpy(packet,"\n");
+	writePkt(sock);
+}
+
+void setupSignals()
 {
 #ifdef SIGSEGV
 	signal(SIGSEGV,sigTrap);
@@ -359,7 +531,14 @@ setupSignals()
 
 
 
-main(argc,argv)
+void usage()
+{
+	printf("Usage :  msql2d [-f ConfFile]\n\n");
+}
+
+
+
+void main(argc,argv)
 	int	argc;
 	char	*argv[];
 {
@@ -367,53 +546,235 @@ main(argc,argv)
 		clientFDs;
 	int	sock,
 		newSock,
+		maxSock,
 		comSock,
 		command,
 		opt,
-		sig,
-		error;
+		error,
+		errFlag = 0,
+		c;
 	char	dbname[30],
 		*uname,
 		*cp,
-		*prog,
 		*arg,
-		path[255];
+		*arg2,
+		path[MAXPATHLEN];
 	FILE	*pidFile;
+        extern  char *optarg;
+	time_t	timeRemain;
+#ifdef HAVE_SETRLIMIT
+	struct	rlimit limit;
+#endif
+
+#ifdef WIN32
+	WORD	wVersion;
+	WSADATA	wsaData;
+#endif
+#if !defined(WIN32)
+	struct	passwd *pwd;
+#endif
+
+	struct	stat sbuf;
+
+
+
+	printf("\n\nMini SQL Version %s\n",SERVER_VERSION);
+	printf("Copyright (c) 1993-94 David J. Hughes\n");
+	printf("Copyright (c) 1995-97 Hughes Technologies Pty Ltd.\n");
+	printf("All rights reserved.\n\n");
 
 	/*
-	** We have enough space for fiddling with the argv, continue
+	** Handle the command line args
 	*/
-	msqlHomeDir = (char *)getenv("MSQL_HOME");
-	if (!msqlHomeDir)
-	{
-		msqlHomeDir = INST_DIR;
+	*confFile = 0;
+        while((c=getopt(argc,argv,"f:"))!= -1)
+        {
+                switch(c)
+                {
+                        case 'f':
+                                if (*confFile)
+                                        errFlag++;
+                                else
+                                        strcpy(confFile,optarg);
+                                break;
+                        case '?':
+                                errFlag++;
+                                break;
+                }
 	}
-	umask(0);
-	numCons=0;
-	chdir(msqlHomeDir);
-	printf("\n\nmSQL Server %s starting ...\n\n",SERVER_VERSION);
+	if (errFlag)
+	{
+		usage();
+		exit(1);
+	}
+
+	if (*confFile == 0)
+	{
+		sprintf(confFile,"%s/msql.conf", INST_DIR);
+	}
+	printf("\tLoading configuration from '%s'.\n",confFile);
+	msqlLoadConfigFile(confFile);
+
+	/*
+	** Check out the file descriptor limit
+	**
+	** Some boxes have broken getrlimit() implementations (or
+	** missing RLIMIT_NOFILE) so we try to use sysconf first
+	**
+	** Watch out for broken BSDI here.  It looks like BSDI returns
+	** the kernel MAX_FILES not the per process MAX_FILE in
+	** getrlimit() which kinda sucks if you want to use select()!
+	*/
+#ifdef HAVE_RLIMIT_NOFILE
+	getrlimit(RLIMIT_NOFILE,&limit);
+	limit.rlim_cur = (limit.rlim_max > 256)? 256 : limit.rlim_max;
+	setrlimit(RLIMIT_NOFILE,&limit);
+	getrlimit(RLIMIT_NOFILE,&limit);
+	maxCons = limit.rlim_cur - CACHE_FDS;
+	printf("\tServer process reconfigured to accept %d connections.\n",
+		maxCons);
+#else
+#  ifdef HAVE_SYSCONF
+	maxCons = sysconf(_SC_OPEN_MAX) - CACHE_FDS;
+	printf("\tServer process reconfigured to accept %d connections.\n",
+		maxCons);
+#  else
+	maxCons = 64 - CACHE_FDS;
+	printf("\tDynamic config failed!  Server can accept %d connections.\n",
+		maxCons);
+#  endif
+#endif
+
+	if (maxCons > 254)
+		maxCons = 254;
+
+	/*
+	** We have to do this now.  If msql is configured to run as nobody
+	** then nobody would have to have write perms for the INST_DIR
+	** This way we can start as root and then run as the MSQL_USER when
+	** we're setup properly
+	*/
+#ifdef WIN32
+	wVersion = MAKEWORD(1,1);
+	if (WSAStartup(wVersion, &wsaData) != 0)
+	{
+		printf("Can't initialise WinSOCK!\n\n");
+		exit(1);
+	}
+	_fmode = _O_BINARY;  /* binary file access is default */
+#endif
 	initDebug();
 	initNet();
 	initServer();
 	initBackend();
-	(void)sprintf(path,"%s/msqld.pid",PID_DIR);
+	readOnly = msqlGetIntConf("system","read_only");
+	(void)sprintf(path,"%s", (char *)msqlGetCharConf("general","pid_file"));
 	pidFile = fopen(path,"w");
 	if (!pidFile)
 	{
-		perror("Couldn't open PID file");
+		perror("Warning : Couldn't open PID file");
 	}
 	else
 	{
-		fprintf(pidFile,"%d",getpid());
+		fprintf(pidFile,"%d", (int)getpid());
 		fclose(pidFile);
 	}
-	chmod(path,0644);
+
+	/*
+	** Are we running as the right user?
+	*/
+#ifndef WIN32
+	pwd = getpwuid(getuid());
+	if (!pwd)
+	{
+		printf("\nError!  No username for our UID (%d)\n\n",
+			getuid());
+		exit(1);
+	}
+	uname = (char *)msqlGetCharConf("general","msql_user");
+	if (strcmp(uname,pwd->pw_name) != 0)
+	{
+		pwd = getpwnam(uname);
+		if (!pwd)
+		{
+			printf("\nError!  Unknown username (%s)\n\n", uname);
+			exit(1);
+		}
+		if (setuid(pwd->pw_uid) < 0)
+		{
+			printf("\nError!  Can't run as user '%s'\n\n",uname);
+			exit(1);
+		}
+	}
+	printf("\tServer running as user '%s'.\n",uname);
+#endif
+	printf("\tServer mode is %s.\n",readOnly?"Read-Only":"Read/Write");
+
+	/*
+	** Ensure that the correct user owns the database files
+	*/
+	sprintf(path,"%s/msqldb",(char *)msqlGetCharConf("general","inst_dir"));
+	if (stat(path,&sbuf) < 0)
+	{
+		printf("\nError!  Can't stat '%s'\n\n",path);
+		exit(1);
+	}
+
+#if !defined(WIN32) && !defined(OS2)
+	if (sbuf.st_uid != getuid())
+	{
+		printf("\nError!  '%s' is not owned by '%s'\n\n",path,
+			pwd->pw_name);
+		exit(1);
+	}
+#endif
+
+	/*
+	** OK, on with the show
+	*/
+
+	yytext = NULL;
 	umask(0);
+	numCons=0;
+	maxSock = 0;
+	msqlHomeDir = (char *)strdup((char *)msqlGetCharConf("general",
+		"inst_dir"));
+	chdir(msqlHomeDir);
+	cleanTmpDir();
+
+#	ifdef BETA_RELEASE
+	timeRemain = (863060000 + 60*24*60*60) - time(NULL);
+	if (timeRemain < 0)
+	{
+		printf("\nThis Beta test release has expired!\n");
+		printf("Check out www.Hughes.com.au for a new release!\n\n");
+		exit(1);
+	}
+	else
+	{
+		printf("\n\tThis Beta test release expires in %d days.\n\n",
+			(int)(timeRemain / (24*60*60)));
+	}
+#	endif
+
+	chmod(path,0700);
+	umask(0077);
 	setupSignals();
 	msqlLoadAcl(1);
 	(void)bzero(&clientFDs,sizeof(fd_set));
 	(void)bzero(conArray,sizeof(conArray));
-	msqlDebug(MOD_GENERAL,"miniSQL debug mode.  Waiting for connections.\n");
+	msqlDebug(MOD_GENERAL,
+		"miniSQL debug mode.  Waiting for connections.\n");
+
+#if !defined(WIN32) && !defined(OS2)
+	msyncTimer = msqlGetIntConf("system","msync_timer");
+	if (msyncTimer > 0)
+	{
+		signal(SIGALRM, mmapSyncer);
+		alarm(msyncTimer);
+	}
+#endif
+
 	while(1)
 	{
 		(void)bcopy(&clientFDs,&readFDs,sizeof(fd_set));
@@ -421,7 +782,7 @@ main(argc,argv)
 #ifdef HAVE_SYS_UN_H
 		FD_SET(UNIXsock,&readFDs);
 #endif
-		if(select(MAX_SOCK_NUM+1,&readFDs,0,0,0) < 0)
+		if(select((maxCons+CACHE_FDS),&readFDs,0,0,0) < 0)
 			continue;
 
 		/*
@@ -443,9 +804,10 @@ main(argc,argv)
 		{
 			struct	sockaddr_in 	cAddr;
 			struct	sockaddr	dummy;
-			int	cAddrLen,
+			size_t	cAddrLen,
 				dummyLen;
 
+			bzero(&cAddr, sizeof(cAddr));
 			cAddrLen = sizeof(struct sockaddr_in);
 			newSock = accept(sock, (struct sockaddr *)&cAddr, 
 				&cAddrLen);
@@ -454,6 +816,8 @@ main(argc,argv)
 				perror("Error in accept ");
 				continue;
 			}
+			if (newSock > maxSock)
+				maxSock = newSock;
 			dummyLen = sizeof(struct sockaddr);
 			if (getsockname(newSock,&dummy, &dummyLen) < 0)
 			{
@@ -472,7 +836,7 @@ main(argc,argv)
 			** Are we over the connection limit
 			*/
 			numCons++;
-			if (numCons > MAX_CON)
+			if (numCons > maxCons)
 			{
 				numCons--;
 				sendError(newSock,CON_COUNT_ERROR);
@@ -486,19 +850,12 @@ main(argc,argv)
 			** store the connection details
 			*/
 
-			if (debugSet(MOD_GENERAL))
-				fprintf(stderr,"\n");
-			if (debugSet(MOD_GENERAL) && !debugSet(MOD_QUERY))
-			{
-				fprintf(stderr, "\n");
-				msqlDebug(MOD_GENERAL,"\n%s\n", timestamp());
-			}
 			msqlDebug(MOD_GENERAL,"New connection received on %d\n",
 				newSock);
 			error = 0;
 			if (sock == IPsock)
 			{
-				int	addrLen;
+				size_t	addrLen;
 				struct	hostent *hp;
 
 				addrLen = sizeof(struct sockaddr);
@@ -507,22 +864,44 @@ main(argc,argv)
 				addrLen = sizeof(struct sockaddr);
 				getsockname(newSock, (struct sockaddr *)
 					&conArray[newSock].local, &addrLen);
-				hp = (struct hostent *)gethostbyaddr(
-				    (char *)&conArray[newSock].remote.sin_addr,
-				    sizeof(conArray[newSock].remote.sin_addr),
-				    AF_INET);
+				if (msqlGetIntConf("system","host_lookup") == 1)
+				{
+					hp = (struct hostent *)gethostbyaddr(
+						(char *)&conArray[newSock].remote.sin_addr,
+						sizeof(conArray[newSock].remote.sin_addr),
+						AF_INET);
+				}
+				else
+				{
+					hp = NULL;
+				}
+
 				if (!hp)
 				{
+					if (bcmp(&conArray[newSock].local,
+							&conArray[newSock].remote,addrLen)== 0)
+					{
+						conArray[newSock].host = (char *)
+							strdup("localhost");
+					}
+					else
+					{
+						conArray[newSock].host = (char *)
+							strdup("UNKNOWN_HOST");
+						msqlDebug(MOD_GENERAL,"Host = UNKNOWN_HOST\n");
+					}
+
+					/*
 					sendError(newSock, BAD_HOST_ERROR);
 					error = 1;
 					shutdown(newSock,2);
 					close(newSock);
-					numCons --;
+					*/
 				}
 				else
 				{
 					conArray[newSock].host = (char *)
-						strdup(hp->h_name);
+						strdup((char *)hp->h_name);
 					msqlDebug(MOD_GENERAL,"Host = %s\n",
 						hp->h_name);
 				}
@@ -537,6 +916,8 @@ main(argc,argv)
 				msqlDebug(MOD_GENERAL,"Host = UNIX domain\n");
 			}
 
+			conArray[newSock].connectTime = time(NULL);
+			conArray[newSock].numQueries = 0;
 			setConnectionState(newSock,&clientFDs);
 
 			if (!error)
@@ -558,7 +939,7 @@ main(argc,argv)
                                         	sizeof(struct sockaddr));
                                 	bzero(&conArray[newSock].
                                         	remote,sizeof(struct sockaddr));
-					numCons--;
+
 				}
 				else
 				{
@@ -581,12 +962,14 @@ main(argc,argv)
 		** This must be a command.
 		*/	
 
-		comSock = 3;
-		while(comSock < MAX_SOCK_NUM)
+		comSock = 0;
+		while(comSock <= maxSock)
 		{
 		    if (FD_ISSET(comSock,&readFDs))
 		    {
 			setConnectionState(comSock,&clientFDs);
+			conArray[comSock].lastQuery = time(NULL);
+			conArray[comSock].numQueries++;
 			if (readPkt(comSock) <= 0)
 			{
 				msqlDebug(MOD_GENERAL,
@@ -606,7 +989,8 @@ main(argc,argv)
 				cp=(char *)strtok(packet+2,"\n\r");
 				if (!cp)
 				{
-					sendError(comSock,NO_DB_ERROR);
+					sendError(comSock, 
+						NO_DB_ERROR);
 					break;
 				}
 				strcpy(dbname,cp);
@@ -642,10 +1026,7 @@ main(argc,argv)
 				cp=(char *)(packet+2);
 				arg = (char *)strdup(cp);
 				if (debugSet(MOD_QUERY))
-				{
-					fprintf(stderr, "\n");
-					msqlDebug(MOD_QUERY,"%s\n", timestamp());
-				}
+					fprintf(stderr,"\n");
 				msqlDebug(MOD_QUERY,"Query = %s",arg);
 				msqlSetDB(conArray[comSock].db);
 				msqlSetPerms(conArray[comSock].access);
@@ -655,7 +1036,23 @@ main(argc,argv)
 
 			    case DB_LIST:
 				curSock = comSock;
-				msqlListDBs(comSock);
+				msqlServerListDBs(comSock);
+				break;
+
+			    case SEQ_INFO:
+				if (!conArray[comSock].db)
+				{
+					sendError(comSock,NO_DB_ERROR);
+					break;
+				}
+				cp = (char *)index(packet,':');
+				cp=(char *)strtok(cp+1,
+					"\n\r");
+				arg = (char *)strdup(cp);
+				curSock = comSock;
+				msqlServerSequenceInfo(comSock,
+					arg,conArray[comSock].db);
+				safeFree(arg);
 				break;
 
 			    case TABLE_LIST:
@@ -665,7 +1062,8 @@ main(argc,argv)
 					break;
 				}
 				curSock = comSock;
-				msqlListTables(comSock,conArray[comSock].db);
+				msqlServerListTables(comSock,
+					conArray[comSock].db);
 				break;
 
 			    case FIELD_LIST:
@@ -676,30 +1074,44 @@ main(argc,argv)
 				}
 				cp=(char *)strtok(packet+2,
 					"\n\r");
-				if (!cp)
-				{
-					sendError(comSock,NO_TABLE_ERROR);
-					break;
-				}
 				arg = (char *)strdup(cp);
 				curSock = comSock;
-				msqlListFields(comSock,
+				msqlServerListFields(comSock,
 					arg,conArray[comSock].db);
 				safeFree(arg);
 				break;
 
+			    case INDEX_LIST:
+				if (!conArray[comSock].db)
+				{
+					sendError(comSock,NO_DB_ERROR);
+					break;
+				}
+				cp=(char *)strtok(packet+2,
+					":\n\r");
+				arg = (char *)strdup(cp);
+				cp=(char *)strtok(NULL,"\n\r");
+				arg2 = (char *)strdup(cp);
+				curSock = comSock;
+				msqlServerListIndex(comSock,
+					arg2,arg,conArray[comSock].db);
+				safeFree(arg);
+				safeFree(arg2);
+				break;
+
 			    case QUIT:
 				msqlDebug(MOD_GENERAL,"DB QUIT!\n");
 				FD_CLR(comSock,&clientFDs);
 				shutdown(comSock,2);
 				close(comSock);
-				if (conArray[comSock].db)
+				if (conArray[comSock].user)
 				{
 				   safeFree(conArray[comSock].db);
 				   safeFree(conArray[comSock].host);
 				   safeFree(conArray[comSock].user);
 				}
 				conArray[comSock].db = NULL;
+				conArray[comSock].user = NULL;
 				numCons--;
 				break;
 		
@@ -711,14 +1123,39 @@ main(argc,argv)
 				}
 				cp=(char *)strtok(packet+2,
 					"\n\r");
-				if (!cp)
+				arg = (char *)strdup(cp);
+				msqlServerCreateDB(comSock,arg);
+				safeFree(arg);
+				break;
+
+			    case COPY_DB:
+				if (!msqlCheckLocal(conArray + comSock))
 				{
-					sendError(comSock,NO_DB_ERROR);
+					sendError(comSock,PERM_DENIED_ERROR);
 					break;
 				}
+				cp=(char *)strtok(packet+2, ":\n\r");
 				arg = (char *)strdup(cp);
-				msqlCreateDB(comSock,arg);
+				cp=(char *)strtok(NULL, "\n\r");
+				arg2 = (char *)strdup(cp);
+				msqlServerCopyDB(comSock,arg, arg2);
 				safeFree(arg);
+				safeFree(arg2);
+				break;
+
+			    case MOVE_DB:
+				if (!msqlCheckLocal(conArray + comSock))
+				{
+					sendError(comSock,PERM_DENIED_ERROR);
+					break;
+				}
+				cp=(char *)strtok(packet+2, ":\n\r");
+				arg = (char *)strdup(cp);
+				cp=(char *)strtok(NULL, "\n\r");
+				arg2 = (char *)strdup(cp);
+				msqlServerMoveDB(comSock,arg, arg2);
+				safeFree(arg);
+				safeFree(arg2);
 				break;
 
 			    case DROP_DB:	
@@ -729,13 +1166,8 @@ main(argc,argv)
 				}
 				cp=(char *)strtok(packet+2,
 					"\n\r");
-				if (!cp)
-				{
-					sendError(comSock,NO_DB_ERROR);
-					break;
-				}
 				arg = (char *)strdup(cp);
-				msqlDropDB(comSock,arg);
+				msqlServerDropDB(comSock,arg);
 				safeFree(arg);
 				break;
 
@@ -746,7 +1178,7 @@ main(argc,argv)
 					break;
 				}
 				(void)sprintf(packet,"-100:\n");
-				reloadAcls(comSock);
+				msqlServerReloadAcls(comSock);
 				writePkt(comSock);
 				break;
 
@@ -762,6 +1194,12 @@ main(argc,argv)
 				exit(0);
 				break;
 
+			    case SERVER_STATS:
+				sendServerStats(comSock);
+				sprintf(packet,"-100:\n");
+				writePkt(comSock);
+				break;
+
 			    default:
 				sendError(comSock, UNKNOWN_COM_ERROR);
 				break;
