@@ -26,15 +26,13 @@
 **		* STATUS: Public domain                              *
 **		*                                                    *
 **		******************************************************
-**
-** Sat Oct 26 14:48:55 MET DST 1996: Martin Schulze <joey@infodrom.north.de>
-**	Implemented a possibility to remove single fields using -r switch.
-**	You may alternativly insert new fields using the -i switch.
-**
-**	As I wanted to keep a patch small, indention is partially strange.
 */
 
 #include <stdio.h>
+#include <string.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <time.h>
 #include <common/portability.h>
 #include "msql.h"
 
@@ -46,25 +44,15 @@
 #define EX_CONSCHECK 3
 
 char	usage[] = "\n\n\
-usage: 	%s [-h host] [-vtdc] database [table]\n\n\
-	Produce an ASCII dump of a database table or an entire database\n\n\
-	-h	Use mSQL server on remote host\n\
-	-v	Verbose mode\n\
-	-t	Do not produce the table creation information\n\
-	-d	Do not produce the data insertion information\n\
-	-c	Complete INSERT statements (including field names)\n\
-	-i	Insert len fields beginning at offs\n\
-	-r	Remove len fields beginning at offs\n\
-\n";
+usage: 	%s [-h host] [-f conf] [-v] [-c] database [table]\n\n\
+	Produce an ASCII dump of a database table or an entire database\n\n";
 
 int	verbose = 0,
-	tFlag = 0,
-	cFlag = 0,
-	dFlag = 0;
+	cFlag = 0;
 int	sock = -1;
-char	insert_pat[5 * 1024];
-int	ioffset = -1, ilength = 0;	/* inserts */
-int	roffset = -1, rlength = 0;	/* removes */
+char	insert_pat[5 * 1024],
+	colDef[1024];
+
 
 
 /*
@@ -174,11 +162,12 @@ int getTableStructure(table)
 	char	*table;
 {
 	m_field 	*mf;
-	m_result	*tableRes;
+	m_result	*tableRes,
+			*indexRes;
+	m_row		row;
+	m_seq		*seq;
 	int		init = 1,
-			count = 0,
-			numFields;
-	char		name[10];
+			numFields = 0;
 
 	if (verbose)
 	{
@@ -191,82 +180,113 @@ int getTableStructure(table)
 		exit(EX_MSQLERR);
 	}
 
-	if (!tFlag)
-	{
-		printf("\n#\n# Table structure for table '%s'\n#\n",table);
-		printf("CREATE TABLE %s (\n", table);
-	}
+	printf("\n#\n# Table structure for table '%s'\n#\n",table);
+	sprintf(insert_pat, "INSERT INTO %s %%s VALUES (", table);
+	printf("CREATE TABLE %s (\n", table);
 
-	if (cFlag)
-		sprintf(insert_pat, "INSERT INTO %s (", table);
-	else
-		sprintf(insert_pat, "INSERT INTO %s VALUES (", table);
-
-	for (count=1; count <= msqlNumFields(tableRes)+ilength; count++)
+	strcpy(colDef,"(");
+	while((mf=msqlFetchField(tableRes)))
 	{
+		if (mf->type > LAST_REAL_TYPE)
+		{
+			break;
+		}
 		if (init)
 		{
-		    if ((roffset == -1) || (roffset > 1) || (count >= roffset+rlength))
 			init = 0;
-		    if (ioffset > msqlNumFields(tableRes)+1)
-		        ioffset = msqlNumFields(tableRes)+1;
 		}
 		else
 		{
-			if (!tFlag && !( !(roffset == -1) && (count >= roffset) && (count < roffset+rlength) ) )
-				printf(",\n");
-			if (cFlag && !( !(roffset == -1) && (count >= roffset) && (count < roffset+rlength) ))
-				strcat(insert_pat,", ");
+			strcat(colDef,", ");
+			printf(",\n");
 		}
-
-		if ( (ioffset != -1) && ( (count >= ioffset) && (count < ioffset+ilength) ) )
+		numFields++;
+		strcat(colDef, mf->name);
+		printf("  %s ", mf->name);
+		switch(mf->type) 
 		{
-			if (!tFlag)
-				printf("  dummy%d CHAR(10)", count);
-			if (cFlag)
-			{
-				sprintf(name, "dummy%d", count);
-				strcat(insert_pat, name);
-			}
-
-		} else {
-			mf=msqlFetchField(tableRes);
-			if (cFlag && !( !(roffset == -1) && (count >= roffset) && (count < roffset+rlength) ))
-				strcat(insert_pat,mf->name);
-		if (!tFlag && !( !(roffset == -1) && (count >= roffset) && (count < roffset+rlength) ))
-		{
-			printf("  %s ", mf->name);
-			switch(mf->type) 
-			{
-				case INT_TYPE:
-					printf("INT");
-					break;
-				case CHAR_TYPE:
-					printf("CHAR(%d)", mf->length);
-					break;
-				case REAL_TYPE:
-					printf("REAL");
-					break;
-				default:
-					fprintf(stderr, 
-						"Unknown field type: %d\n", 
-						mf->type);
-					exit(EX_CONSCHECK);
-			}
-
-			if(IS_NOT_NULL(mf->flags) )
-				printf(" NOT NULL");
-			if(IS_PRI_KEY(mf->flags) )
-				printf(" PRIMARY KEY");
-		}
+			case INT_TYPE:
+				printf("INT");
+				break;
+			case UINT_TYPE:
+				printf("UINT");
+				break;
+			case MONEY_TYPE:
+				printf("MONEY");
+				break;
+			case TIME_TYPE:
+				printf("TIME");
+				break;
+			case DATE_TYPE:
+				printf("DATE");
+				break;
+			case CHAR_TYPE:
+				printf("CHAR(%d)", mf->length);
+				break;
+			case REAL_TYPE:
+				printf("REAL");
+				break;
+			case TEXT_TYPE:
+				printf("TEXT(%d)", mf->length);
+				break;
+			default:
+				fprintf(stderr,
+					"Unknown field type: %d\n", 
+					mf->type);
+				exit(EX_CONSCHECK);
 		}
+		if(IS_NOT_NULL(mf->flags) )
+			printf(" NOT NULL");
 	}
-	if (!tFlag)
+	printf("\n) \\g\n\n");
+	strcat(colDef,")");
+
+	while(mf)
+	{
+		char	*type;
+
+		if (mf->type != IDX_TYPE)
+		{
+			mf = msqlFetchField(tableRes);
+			continue;
+		}
+		indexRes = msqlListIndex(sock,table,mf->name);
+		row = msqlFetchRow(indexRes);
+		if (strcmp(row[0],"avl") != 0)
+			type = row[0];
+		else
+			type = NULL;
+		printf("CREATE %s %s INDEX %s ON %s (",
+			IS_UNIQUE(mf->flags)?"UNIQUE":"",
+			type?type:"", mf->name, table);
+		row = msqlFetchRow(indexRes);
+		init = 1;
+		while(row)
+		{
+			if (init)
+			{
+				init=0;
+				printf("\n");
+			}
+			else
+			{
+				printf(",\n");
+			}
+			printf("\t%s",row[0]);
+			row = msqlFetchRow(indexRes);
+		}
 		printf("\n) \\g\n\n");
-	if (cFlag)
-		strcat(insert_pat,") VALUES (");
-	numFields = msqlNumFields(tableRes);
+		msqlFreeResult(indexRes);
+		mf = msqlFetchField(tableRes);
+	}
 	msqlFreeResult(tableRes);
+
+	seq = msqlGetSequenceInfo(sock,table);
+	if (seq)
+	{
+		printf("CREATE SEQUENCE ON %s STEP %d VALUE %d \\g\n\n",
+			table, seq->step, seq->value);
+	}
 	return(numFields);
 }
 
@@ -288,7 +308,6 @@ void dumpTable(numFields,table)
 	m_row 		row;
 	int		i;
 	int		init = 1;
-	int		count;
 
 	if (verbose)
 		fprintf(stderr, "Sending SELECT query...\n");
@@ -309,36 +328,28 @@ void dumpTable(numFields,table)
 		exit(EX_CONSCHECK);
 	}
 
-	while (row=msqlFetchRow(res)) 
+	while ((row=msqlFetchRow(res)))
 	{
-		printf("%s", insert_pat);
+		printf(insert_pat, cFlag?colDef:"");
 		init = 1;
-		i = 0;
 		msqlFieldSeek(res,0);
-		for (count = 1; count <= msqlNumFields(res)+ilength; count++) 
+		for (i = 0; i < msqlNumFields(res); i++) 
 		{
-			if (!init) {
-			    if ( !( !(roffset == -1) && (count >= roffset) && (count < roffset+rlength) ) )
-				printf(",");
-			} else {
-			    if ((roffset == -1) || (roffset > 1) || (count >= roffset+rlength))
-				init=0;
-			}
-
-		if ( (ioffset != -1) && ( (count >= ioffset) && (count < ioffset+ilength) ) )
-		{
-			printf("NULL");
-		} else {
-		    if ( !( !(roffset == -1) && (count >= roffset) && (count < roffset+rlength) ) )
-		    {
 			if (!(field = msqlFetchField(res))) 
 			{
 				fprintf(stderr,"Not enough fields! Aborting\n");
 				exit(EX_CONSCHECK);
 			}
+			if (!init )
+				printf(",");
+			else
+				init=0;
 			if (row[i])
 			{
-				if (field->type == CHAR_TYPE)
+				if (field->type == CHAR_TYPE || 
+				    field->type == TEXT_TYPE ||
+				    field->type == DATE_TYPE ||
+				    field->type == TIME_TYPE )
 				{
 					tmp = escapeText(row[i]);
 					printf("\'%s\'", tmp);
@@ -353,9 +364,6 @@ void dumpTable(numFields,table)
 			{
 				printf("NULL");
 			}
-		    }
-		    i++;
-		}
 		}
 		printf(")\\g\n");
 	}
@@ -389,7 +397,7 @@ char *getTableName()
 
 
 
-main(argc, argv)
+int main(argc, argv)
 	int	argc;
 	char	*argv[];
 {
@@ -398,14 +406,15 @@ main(argc, argv)
 		errFlag = 0;
 	char	*host = NULL,
 		*database = NULL,
-		*table = NULL;
+		*table = NULL,
+		*confFile = NULL;
 	extern	char *optarg;
 	extern	int optind;
 
 	/*
 	** Check out the args
 	*/
-	while((c=getopt(argc,argv,"h:i:vtcdr:"))!= -1)
+	while((c=getopt(argc,argv,"h:f:vc"))!= -1)
 	{
 		switch(c)
 		{
@@ -416,40 +425,6 @@ main(argc, argv)
 					host = optarg;
 				break;
 
-			case 'i':
-				if (ioffset != -1)
-					errFlag++;
-				else
-					ioffset = atoi(optarg);
-				if (argv[optind])
-				{
-					ilength = atoi(argv[optind]);
-					if (ilength)
-						optind++;
-					else
-						ilength = 1;
-				}
-				else
-					ilength = 1;
-				break;
-
-			case 'r':
-				if (roffset != -1)
-					errFlag++;
-				else
-					roffset = atoi(optarg);
-				if (argv[optind])
-				{
-					rlength = atoi(argv[optind]);
-					if (rlength)
-						optind++;
-					else
-						rlength = 1;
-				}
-				else
-					rlength = 1;
-				break;
-
 			case 'v':
 				if (verbose)
 					errFlag++;
@@ -457,26 +432,19 @@ main(argc, argv)
 					verbose++;
 				break;
 
-			case 't':
-				if (tFlag)
-					errFlag++;
-				else
-					tFlag++;
-				break;
+                        case 'f':
+                                if (confFile)
+                                        errFlag++;
+                                else
+                                        confFile = optarg;
+                                break;
 
-			case 'c':
-				if (cFlag)
-					errFlag++;
-				else
-					cFlag++;
-				break;
-
-			case 'd':
-				if (dFlag)
-					errFlag++;
-				else
-					dFlag++;
-				break;
+                        case 'c':
+                                if (cFlag)
+                                        errFlag++;
+                                else
+                                        cFlag++;
+                                break;
 
 			case '?':
 				errFlag++;
@@ -493,31 +461,21 @@ main(argc, argv)
 	if (optind < argc)
 		table = argv[optind++];
 
-	if (!table && ((roffset != -1) || (ioffset != -1)))
-	{
-		fprintf(stderr, "Inserts or removes can only be applied to one single table.\n");
-		ioffset = -1;
-		roffset = -1;
-	}
-	if ((ioffset != -1) && (ioffset < 1))
-		ioffset = 1;
-	if ((roffset != -1) && (roffset < 1))
-		roffset = 1;
-	if ((ioffset != -1) && (roffset != -1))
-	{
-		fprintf(stderr,"Don't use -i and -r in one call.\n");
-		fprintf(stderr,"-i %d, -r %d\n", ioffset, roffset);
-		exit(EX_USAGE);
-	}
-
 	if (!database)
 	{
 		fprintf(stderr,usage, argv[0]);
 		exit(EX_USAGE);
 	}
 	
+        /*
+        ** If we have a config file override the default config
+        */
+        if (confFile)
+        {
+                msqlLoadConfigFile(confFile);
+        }
 
-	printf("#\n# mSQL Dump  (requires mSQL-1.0.6 or better)\n#\n");
+	printf("#\n# mSQL Dump  (requires mSQL 2.0 Beta 5 or newer)\n#\n");
 	printf("# Host: %s    Database: %s\n",
 	    host ? host : "localhost", database);
 	printf("#--------------------------------------------------------\n\n");
@@ -525,16 +483,14 @@ main(argc, argv)
 	if (table)
 	{
 		numRows = getTableStructure(table);
-		if (!dFlag)
-			dumpTable(numRows,table);
+		dumpTable(numRows,table);
 	}
 	else
 	{
 		while((table = getTableName()))
 		{
 			numRows = getTableStructure(table);
-			if (!dFlag)
-				dumpTable(numRows,table);
+			dumpTable(numRows,table);
 		}
 	}
 	dbDisconnect(host);
