@@ -24,6 +24,13 @@
 #include <arpa/inet.h>
 #include <signal.h>
 #include <netdb.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <string.h>
+
+#ifdef WIN32
+#  include <winsock.h>
+#endif
 
 #include <common/portability.h>
 #include "msql_priv.h"
@@ -34,7 +41,16 @@ static	int	readTimeout;
 u_char	*packet = NULL;
 
 
-intToBuf(cp,val)
+#ifdef WIN32
+RETSIGTYPE alarmHandler(int);
+
+void alarm(val)
+	int	val;
+{
+}
+#endif
+
+void intToBuf(cp,val)
         u_char  *cp;
         int     val;
 {
@@ -45,7 +61,7 @@ intToBuf(cp,val)
 }
 
 
-bufToInt(cp)
+int bufToInt(cp)
         u_char  *cp;
 {
         int val;
@@ -59,17 +75,16 @@ bufToInt(cp)
 }
 
 
-initNet()
+void initNet()
 {
 	packet = (u_char *)packetBuf + 4;
 }
 
 
 
-void writePkt(fd)
+int writePkt(fd)
 	int	fd;
 {
-	u_char	*cp;
 	int	len,
 		offset,
 		remain,
@@ -81,14 +96,15 @@ void writePkt(fd)
 	remain = len+4;
 	while(remain > 0)
 	{
-		numBytes = write(fd,packetBuf + offset, remain);
+		numBytes = send(fd,packetBuf + offset, remain,0);
 		if (numBytes == -1)
 		{
-			return;
+			return(-1);
 		}
 		offset += numBytes;
 		remain -= numBytes;
 	}
+	return(0);
 }
 
 
@@ -103,15 +119,13 @@ RETSIGTYPE alarmHandler(sig)
 int readPkt(fd)
 	int	fd;
 {
-	u_char	*c,
-		buf[4];
+	u_char	 buf[4];
 	int	len,
 		remain,
 		offset,
 		numBytes;
-	static	int init = 1;
 
-#ifdef MSQL_SERVER
+#if defined(MSQL_SERVER) && !defined(WIN32)
 	if (init)
 	{
 		signal(SIGALRM,alarmHandler);
@@ -124,17 +138,19 @@ int readPkt(fd)
 	offset = 0;
 	while(remain > 0)
 	{
-		numBytes = read(fd,buf + offset,remain);
+		numBytes = recv(fd,buf + offset,remain,0);
 		if(numBytes <= 0)
 		{
+#if defined(MSQL_SERVER) && !defined(WIN32)
 			alarm(0);
+#endif
          		return(-1);
 		}
 		remain -= numBytes;
 		offset += numBytes;
 		
 	}
-#ifdef MSQL_SERVER
+#if defined(MSQL_SERVER) && !defined(WIN32)
 	if (readTimeout)
 	{
 		alarm(0);
@@ -152,7 +168,7 @@ int readPkt(fd)
 	offset = 0;
 	while(remain > 0)
 	{
-		numBytes = read(fd,packet+offset,remain);
+		numBytes = recv(fd,packet+offset,remain,0);
 		if (readTimeout)
 		{
 			alarm(0);
@@ -166,10 +182,10 @@ int readPkt(fd)
 		remain -= numBytes;
 		offset += numBytes;
 	}
-	*(packet+offset) = 0;
 #ifdef MSQL_SERVER
 	alarm(0);
 #endif
+	*(packet + len) = 0;
         return(len);
 }
 
@@ -222,7 +238,7 @@ char	*buf;
 	num <<= HIGH_BITS;
 #endif
 
-	bcopy((char *)&num, buf, 4);
+	bcopy4((char *)&num, buf);
 	return 0;
 }
 
@@ -237,7 +253,7 @@ char	*buf;
 {
 	int	num;
 
-	bcopy(buf, (char *)&num, 4);
+	bcopy4(buf, (char *)&num);
 
 #if BIG_INTS
 	num >>= HIGH_BITS;
